import json
from urllib import parse

from helpers.ConfigHelper import get_config
from helpers.api.utils import url_join
import helpers.api.http_helper as request

created_spaces = {}
user_spaces = {}
space_role = ['manager', 'editor', 'viewer']


def get_space_endpint():
    return url_join(get_config('localBackendUrl'), 'graph', 'v1.0', 'drives')


def get_dav_endpint():
    return url_join(get_config('localBackendUrl'), 'dav', 'spaces')


def get_share_endpint():
    return url_join(
        get_config('localBackendUrl'),
        'ocs/v2.php/apps',
        'files_sharing/api/v1/shares',
    )


def create_space(space_name):
    body = json.dumps({'name': space_name})
    response = request.post(get_space_endpint(), body)
    request.assert_http_status(response, 201, f'Failed to create space {space_name}')
    # save created space
    resp_object = response.json()
    created_spaces[space_name] = resp_object['id']


def fetch_spaces(user=None, query=''):
    if query:
        query = '?' + query
    url = get_space_endpint() + query
    response = request.get(url=url, user=user)
    request.assert_http_status(response, 200, 'Failed to get spaces')
    return response.json()['value']


def get_project_spaces(user=None):
    search_query = '$filter=driveType eq \'project\''
    return fetch_spaces(query=search_query, user=user)


def get_space_id(space_name, user=None):
    spaces = {**created_spaces, **user_spaces}
    if not space_name in spaces.keys():
        return fetch_space_id(space_name, user)
    return spaces.get(space_name)


def fetch_space_id(space_name, user=None):
    spaces = fetch_spaces(user=user)
    space_id = None
    for space in spaces:
        if space['name'] == space_name:
            user_spaces[space_name] = space['id']
            space_id = space['id']
            break
    return space_id


def delete_project_spaces():
    global created_spaces, user_spaces
    for _, space_id in created_spaces.items():
        disable_project_space(space_id)
        delete_project_space(space_id)
    created_spaces = {}
    user_spaces = {}


def disable_project_space(space_id):
    url = url_join(get_space_endpint(), space_id)
    response = request.delete(url)
    request.assert_http_status(response, 204, f'Failed to disable space {space_id}')


def delete_project_space(space_id):
    url = url_join(get_space_endpint(), space_id)
    response = request.delete(url, {'Purge': 'T'})
    request.assert_http_status(response, 204, f'Failed to delete space {space_id}')


def create_space_folder(space_name, folder_name):
    space_id = get_space_id(space_name)
    url = url_join(get_dav_endpint(), space_id, folder_name)
    response = request.mkcol(url)
    request.assert_http_status(
        response,
        201,
        f'Failed to create folder "{folder_name}" in space "{space_name}"',
    )


def create_space_file(space_name, file_name, content):
    space_id = get_space_id(space_name)
    url = url_join(get_dav_endpint(), space_id, file_name)
    response = request.put(url, content)
    if response.status_code not in (201, 204):
        raise AssertionError(
            f"Creating file '{file_name}' in space '{space_name}' failed with {response.status_code}\n"
            + response.text
        )


def add_user_to_space(user, space_name, role):
    role = role.lower()
    if not role in space_role:
        raise ValueError(f"Cannot set the role '{role}' to a space")

    space_id = get_space_id(space_name)
    url = get_share_endpint()
    body = parse.urlencode(
        {
            'space_ref': space_id,
            'shareType': 7,
            'shareWith': user,
            'role': role,
        }
    )
    headers = {'Content-Type': 'application/x-www-form-urlencoded'}
    response = request.post(url, body, headers)
    request.assert_http_status(
        response, 200, f'Failed to add user "{user}" to space "{space_name}"'
    )


def get_file_content(space_name, file_name, user=None):
    space_id = get_space_id(space_name, user)
    url = url_join(get_dav_endpint(), space_id, file_name)
    response = request.get(url=url, user=user)
    request.assert_http_status(response, 200, f'Failed to get file "{file_name}"')
    return response.text


def resource_exists(space_name, resource, user=None):
    space_id = get_space_id(space_name, user)
    url = url_join(get_dav_endpint(), space_id, resource)
    response = request.get(url=url, user=user)
    if response.status_code == 200:
        return True
    return False



# encoding: UTF-8
# fmt: off

from objectmaphelper import *

settings_OCC_SettingsDialog = {"name": "Settings", "type": "OCC::SettingsDialog", "visible": 1}
owncloudWizard_OCC_OwncloudWizard = {"name": "owncloudWizard", "type": "OCC::OwncloudWizard", "visible": 1}
qFileDialog_QFileDialog = {"name": "QFileDialog", "type": "QFileDialog", "visible": 1}
settings_stack_QStackedWidget = {"name": "stack", "type": "QStackedWidget", "visible": 1, "window": settings_OCC_SettingsDialog}
settings_dialogStack_QStackedWidget = {"name": "dialogStack", "type": "QStackedWidget", "visible": 1, "window": settings_OCC_SettingsDialog}
qFileDialog_fileNameLabel_QLabel = {"name": "fileNameLabel", "type": "QLabel", "visible": 1, "window": qFileDialog_QFileDialog}
sharingDialog_OCC_ShareDialog = {"name": "SharingDialog", "type": "OCC::ShareDialog", "visible": 1}
sharingDialog_qt_tabwidget_stackedwidget_QStackedWidget = {"name": "qt_tabwidget_stackedwidget", "type": "QStackedWidget", "visible": 1, "window": sharingDialog_OCC_ShareDialog}
qt_tabwidget_stackedwidget_SharingDialogUG_OCC_ShareUserGroupWidget = {"container": sharingDialog_qt_tabwidget_stackedwidget_QStackedWidget, "name": "SharingDialogUG", "type": "OCC::ShareUserGroupWidget", "visible": 1}
sharingDialogUG_scrollArea_QScrollArea = {"container": qt_tabwidget_stackedwidget_SharingDialogUG_OCC_ShareUserGroupWidget, "name": "scrollArea", "type": "QScrollArea", "visible": 1}
settings_settingsdialog_toolbutton_Quit_ownCloud_QToolButton = {"name": "settingsdialog_toolbutton_Quit ownCloud", "type": "QToolButton", "visible": 1, "window": settings_OCC_SettingsDialog}
settings_settingsdialog_toolbutton_Settings_QToolButton = {"name": "settingsdialog_toolbutton_Settings", "type": "QToolButton", "visible": 1, "window": settings_OCC_SettingsDialog}
stack_qt_tabwidget_stackedwidget_QStackedWidget = {"container": settings_stack_QStackedWidget, "name": "qt_tabwidget_stackedwidget", "type": "QStackedWidget", "visible": 1}
qt_tabwidget_stackedwidget_OCC_IssuesWidget_OCC_IssuesWidget = {"container": stack_qt_tabwidget_stackedwidget_QStackedWidget, "name": "OCC__IssuesWidget", "type": "OCC::IssuesWidget", "visible": 1}
sharingDialog_qt_tabwidget_tabbar_QTabBar = {"name": "qt_tabwidget_tabbar", "type": "QTabBar", "visible": 1, "window": sharingDialog_OCC_ShareDialog}
qt_tabwidget_stackedwidget_OCC_ShareLinkWidget_OCC_ShareLinkWidget = {"container": sharingDialog_qt_tabwidget_stackedwidget_QStackedWidget, "name": "OCC__ShareLinkWidget", "type": "OCC::ShareLinkWidget", "visible": 1}
oCC_ShareLinkWidget_checkBox_password_QCheckBox = {"container": qt_tabwidget_stackedwidget_OCC_ShareLinkWidget_OCC_ShareLinkWidget, "name": "checkBox_password", "type": "QCheckBox", "visible": 1}
oCC_ShareLinkWidget_widget_editing_QWidget = {"container": qt_tabwidget_stackedwidget_OCC_ShareLinkWidget_OCC_ShareLinkWidget, "name": "widget_editing", "type": "QWidget", "visible": 1}
oCC_ShareLinkWidget_checkBox_password_QProgressIndicator = {"aboveWidget": oCC_ShareLinkWidget_widget_editing_QWidget, "container": qt_tabwidget_stackedwidget_OCC_ShareLinkWidget_OCC_ShareLinkWidget, "leftWidget": oCC_ShareLinkWidget_checkBox_password_QCheckBox, "type": "QProgressIndicator", "unnamed": 1, "visible": 1}
oCC_ShareLinkWidget_linkShares_QTableWidget = {"container": qt_tabwidget_stackedwidget_OCC_ShareLinkWidget_OCC_ShareLinkWidget, "name": "linkShares", "type": "QTableWidget", "visible": 1}
oCC_ShareLinkWidget_lineEdit_password_QLineEdit = {"container": qt_tabwidget_stackedwidget_OCC_ShareLinkWidget_OCC_ShareLinkWidget, "name": "lineEdit_password", "type": "QLineEdit", "visible": 1}
oCC_ShareLinkWidget_checkBox_expire_QCheckBox = {"container": qt_tabwidget_stackedwidget_OCC_ShareLinkWidget_OCC_ShareLinkWidget, "name": "checkBox_expire", "type": "QCheckBox", "visible": 1}
oCC_ShareLinkWidget_checkBox_expire_QProgressIndicator = {"aboveWidget": oCC_ShareLinkWidget_lineEdit_password_QLineEdit, "container": qt_tabwidget_stackedwidget_OCC_ShareLinkWidget_OCC_ShareLinkWidget, "leftWidget": oCC_ShareLinkWidget_checkBox_expire_QCheckBox, "type": "QProgressIndicator", "unnamed": 1, "visible": 1}
settings_settingsdialog_toolbutton_Add_account_QToolButton = {"name": "settingsdialog_toolbutton_Add account", "type": "QToolButton", "visible": 1, "window": settings_OCC_SettingsDialog}
settings_settingsdialog_toolbutton_Activity_QToolButton = {"name": "settingsdialog_toolbutton_Activity", "type": "QToolButton", "visible": 1, "window": settings_OCC_SettingsDialog}
disable_virtual_file_support_QMessageBox = {"type": "QMessageBox", "unnamed": 1, "visible": 1, "windowTitle": "Disable virtual file support?"}
owncloudWizard_urlLabel_QLabel = {"name": "urlLabel", "type": "QLabel", "visible": 1, "window": owncloudWizard_OCC_OwncloudWizard}
setupWizardWindow_OCC_Wizard_SetupWizardWindow = {"name": "SetupWizardWidget", "type": "OCC::Wizard::SetupWizardWidget", "visible": 1}
setupWizardWindow_contentWidget_QStackedWidget = {"name": "contentWidget", "type": "QStackedWidget", "visible": 1, "window": setupWizardWindow_OCC_Wizard_SetupWizardWindow}
insecure_connection_QMessageBox = {"type": "QMessageBox", "unnamed": 1, "visible": 1, "windowTitle": "Insecure connection"}
contentWidget_advancedConfigGroupBox_QGroupBox = {"container": setupWizardWindow_contentWidget_QStackedWidget, "name": "advancedConfigGroupBox", "type": "QGroupBox", "visible": 1}
advancedConfigGroupBox_localDirectoryGroupBox_QGroupBox = {"container": contentWidget_advancedConfigGroupBox_QGroupBox, "name": "localDirectoryGroupBox", "type": "QGroupBox", "visible": 1}
advancedConfigGroupBox_syncModeGroupBox_QGroupBox = {"container": contentWidget_advancedConfigGroupBox_QGroupBox, "name": "syncModeGroupBox", "type": "QGroupBox", "visible": 1}
add_Folder_Sync_Connection_OCC_FolderWizard = {"type": "OCC::FolderWizard", "unnamed": 1, "visible": 1, "windowTitle": "Add Folder Sync Connection"}
add_Folder_Sync_Connection_groupBox_QGroupBox = {"name": "groupBox", "type": "QGroupBox", "visible": 1, "window": add_Folder_Sync_Connection_OCC_FolderWizard}
add_Folder_Sync_Connection_Deselect_remote_folders_you_do_not_wish_to_synchronize_QLabel = {"text": "Deselect remote folders you do not wish to synchronize.", "type": "QLabel", "unnamed": 1, "visible": 1, "window": add_Folder_Sync_Connection_OCC_FolderWizard}
folder_Sync_Connection_Deselect_remote_folders_QTreeWidget = {"aboveWidget": add_Folder_Sync_Connection_Deselect_remote_folders_you_do_not_wish_to_synchronize_QLabel, "type": "QTreeWidget", "unnamed": 1, "visible": 1, "window": add_Folder_Sync_Connection_OCC_FolderWizard}
deselect_remote_folders_you_do_not_wish_to_synchronize_ownCloud_QModelIndex = {"column": 0, "container": folder_Sync_Connection_Deselect_remote_folders_QTreeWidget, "type": "QModelIndex"}
loginRequiredDialog_OCC_LoginRequiredDialog = {"name": "LoginRequiredDialog", "type": "OCC::LoginRequiredDialog", "visible": 1}
loginRequiredDialog_contentWidget_QStackedWidget = {"name": "contentWidget", "type": "QStackedWidget", "visible": 1, "window": loginRequiredDialog_OCC_LoginRequiredDialog}
contentWidget_contentWidget_QStackedWidget = {"container": setupWizardWindow_contentWidget_QStackedWidget, "name": "contentWidget", "type": "QStackedWidget", "visible": 1}
add_Folder_Sync_Connection_tableView_QTableView = {"name": "tableView","type": "QTableView","visible": 1,"window": add_Folder_Sync_Connection_OCC_FolderWizard}
stack_scrollArea_QScrollArea = {"container": settings_stack_QStackedWidget, "name": "scrollArea", "type": "QScrollArea", "visible": 1}
stack_stackedWidget_QStackedWidget = {"container": settings_stack_QStackedWidget, "name": "stackedWidget", "type": "QStackedWidget", "visible": 1}
stackedWidget_quickWidget_QQuickWidget = {"container": stack_stackedWidget_QStackedWidget, "name": "quickWidget", "type": "QQuickWidget", "visible": 1}
quickWidget_scrollView_ScrollView = {"container": stackedWidget_quickWidget_QQuickWidget, "id": "scrollView", "type": "ScrollView", "unnamed": 1, "visible": True}
scrollView_ListView = {"container": quickWidget_scrollView_ScrollView, "type": "ListView", "unnamed": 1, "visible": True}
dialogStack_quickWidget_QQuickWidget = {"container": settings_dialogStack_QStackedWidget, "name": "quickWidget", "type": "QQuickWidget", "visible": 1}
create_Remote_Folder_QInputDialog = {"type": "QInputDialog", "unnamed": 1, "visible": 1, "windowTitle": "Create Remote Folder"}
create_Remote_Folder_Enter_the_name_of_the_new_folder_to_be_created_below_QLabel = {"text": "Enter the name of the new folder to be created below '/':", "type": "QLabel", "unnamed": 1, "visible": 1, "window": create_Remote_Folder_QInputDialog}
groupBox_folderTreeWidget_QTreeWidget = {"container": add_Folder_Sync_Connection_groupBox_QGroupBox, "name": "folderTreeWidget", "type": "QTreeWidget", "visible": 1}
confirm_Folder_Sync_Connection_Removal_QMessageBox = {"type": "QMessageBox", "unnamed": 1, "visible": 1, "windowTitle": "Confirm Folder Sync Connection Removal"}
stackedWidget_quickWidget_OCC_QmlUtils_OCQuickWidget = {"container": stack_stackedWidget_QStackedWidget, "name": "quickWidget", "type": "OCC::QmlUtils::OCQuickWidget", "visible": 1}
qt_tabwidget_stackedwidget_OCC_ProtocolWidget_OCC_ProtocolWidget = {"container": stack_qt_tabwidget_stackedwidget_QStackedWidget, "name": "OCC__ProtocolWidget", "type": "OCC::ProtocolWidget", "visible": 1}
oCC_ProtocolWidget_tableView_QTableView = {"container": qt_tabwidget_stackedwidget_OCC_ProtocolWidget_OCC_ProtocolWidget, "name": "_tableView", "type": "QTableView", "visible": 1}
oCC_IssuesWidget_tableView_QTableView = {"container": qt_tabwidget_stackedwidget_OCC_IssuesWidget_OCC_IssuesWidget, "name": "_tableView", "type": "QTableView", "visible": 1}
dialogStack_quickWidget_OCC_QmlUtils_OCQuickWidget = {"container": settings_dialogStack_QStackedWidget, "name": "quickWidget", "type": "OCC::QmlUtils::OCQuickWidget", "visible": 1}
contentWidget_OCC_QmlUtils_OCQuickWidget = {"container": contentWidget_contentWidget_QStackedWidget, "type": "OCC::QmlUtils::OCQuickWidget", "unnamed": 1, "visible": 1}
stackedWidget_Add_Folder_Sync_Connection_QGroupBox = {"container": stack_stackedWidget_QStackedWidget, "title": "Add Folder Sync Connection", "type": "QGroupBox", "unnamed": 1, "visible": 1}
stackedWidget_groupBox_QGroupBox = {"container": settings_stack_QStackedWidget, "name": "groupBox", "type": "QGroupBox", "visible": 1}
groupBox_OCC_QmlUtils_OCQuickWidget = {"container": stackedWidget_groupBox_QGroupBox, "type": "OCC::QmlUtils::OCQuickWidget", "unnamed": 1, "visible": 1}



from helpers.api import oc10 as oc
from helpers.api import ocis
from helpers.ConfigHelper import get_config, is_owncloud_client
from helpers import UserHelper

created_groups = {}
created_users = {}


def create_group(group_name):
    if get_config('ocis'):
        group_info = ocis.create_group(group_name)
    else:
        group_info = oc.create_group(group_name)
    created_groups[group_name] = group_info


def delete_created_groups():
    for group_name, group_info in list(created_groups.items()):
        if get_config('ocis'):
            ocis.delete_group(group_info['id'])
        else:
            oc.delete_group(group_info['id'])
        del created_groups[group_name]


def add_user_to_group(user, group_name):
    if get_config('ocis'):
        ocis.add_user_to_group(user, group_name)
    else:
        oc.add_user_to_group(user, group_name)


def create_user(username):
    if not is_owncloud_client():
        user = UserHelper.test_users[username]
        created_users[username] = {
            'username': user.username,
            'password': user.password,
            'displayname': user.displayname,
            'email': user.email,
        }
        return

    if username in UserHelper.test_users:
        user = UserHelper.test_users[username]
    else:
        user = UserHelper.User(
            username=username,
            displayname=username,
            email=f'{username}@mail.com',
            password=UserHelper.get_default_password(),
        )

    if get_config('ocis'):
        user_info = ocis.create_user(
            user.username,
            user.password,
            user.displayname,
            user.email,
        )
    else:
        user_info = oc.create_user(
            user.username,
            user.password,
            user.displayname,
            user.email,
        )
    created_users[username] = user_info


def delete_created_users():
    for username, user_info in list(created_users.items()):
        if is_owncloud_client():
            if get_config('ocis'):
                ocis.delete_user(user_info['id'])
            else:
                oc.delete_user(user_info['id'])
        del created_users[username]



from helpers.SpaceHelper import (
    create_space,
    create_space_folder,
    create_space_file,
    add_user_to_space,
    get_file_content,
    resource_exists,
)
from helpers.ConfigHelper import is_owncloud_client
import helpers.api.external_api as ext


@Given('the administrator has created a space "|any|"')
def step(context, space_name):
    if not is_owncloud_client():
        ext.create_space(space_name)
    else:
        create_space(space_name)


@Given('the administrator has created a folder "|any|" in space "|any|"')
def step(context, folder_name, space_name):
    if not is_owncloud_client():
        ext.create_space_folder(space_name, folder_name)
    else:
        create_space_folder(space_name, folder_name)


@Given(
    'the administrator has uploaded a file "|any|" with content "|any|" inside space "|any|"'
)
def step(context, file_name, content, space_name):
    if not is_owncloud_client():
        ext.upload_space_file(space_name, file_name, content)
    else:
        create_space_file(space_name, file_name, content)


@Given('the administrator has added user "|any|" to space "|any|" with role "|any|"')
def step(context, user, space_name, role):
    if not is_owncloud_client():
        ext.add_user_to_space(user, space_name, role)
    else:
        add_user_to_space(user, space_name, role)


@Then(
    'as "|any|" the file "|any|" in the space "|any|" should have content "|any|" in the server'
)
def step(context, user, file_name, space_name, content):
    if not is_owncloud_client():
        downloaded_content = ext.get_file_content(user, file_name, space_name)
    else:
        downloaded_content = get_file_content(space_name, file_name, user)
    test.compare(downloaded_content, content, 'Comparing file content')


@Then(
    r'as "([^"]*)" the space "([^"]*)" (should|should not) have (?:folder|file) "([^"]*)" in the server',
    regexp=True,
)
def step(context, user, space_name, should_or_should_not, resource_name):
    expected = should_or_should_not == 'should'
    if not is_owncloud_client():
        exists = ext.resource_exists(user, resource_name, space_name)
    else:
        exists = resource_exists(space_name, resource_name, user)
    test.compare(exists, expected, 'Resource exists')


@When('user "|any|" uploads a file "|any|" with content "|any|" inside space "|any|"')
def step(context, user, file_name, file_content, space_name):
    ext.upload_space_file(space_name, file_name, file_content, user)



import requests
import urllib3

from helpers.UserHelper import basic_auth_header
from helpers.ConfigHelper import get_config

urllib3.disable_warnings()


def send_request(
    url, method, body=None, headers=None, user=None, password=None, files=None
):
    auth_header = basic_auth_header(user, password)
    if not headers:
        headers = {}
    headers.update(auth_header)
    return requests.request(
        method,
        url,
        data=body,
        headers=headers,
        verify=False,
        # in seconds
        # e.g.: 60
        timeout=get_config("maxSyncTimeout"),
        files=files,
    )


def get(url, headers=None, user=None, password=None):
    return send_request(
        url=url, method="GET", headers=headers, user=user, password=password
    )


def post(url, body=None, headers=None, user=None, files=None):
    return send_request(url, "POST", body, headers, user, files=files)


def put(url, body=None, headers=None, user=None):
    return send_request(url, "PUT", body, headers, user)


def delete(url, headers=None, user=None):
    return send_request(url=url, method="DELETE", headers=headers, user=user)


def mkcol(url, headers=None, user=None):
    return send_request(url=url, method="MKCOL", headers=headers, user=user)


def propfind(url, body=None, headers=None, user=None, password=None):
    return send_request(url, "PROPFIND", body, headers, user, password)


def assert_http_status(response, expected_code, message=""):
    response_body = ""
    if response.text:
        response_body = response.text

    assert (
        response.status_code == expected_code
    ), f"{message}\nRequest failed with status code '{response.status_code}'\n{response_body}"



from helpers.FilesHelper import get_file_size_on_disk, get_file_size
from helpers.SetupClientHelper import get_resource_path


@Then('the placeholder of file "|any|" should exist on the file system')
def step(context, file_name):
    resource_path = get_resource_path(file_name)
    size_on_disk = get_file_size_on_disk(resource_path)
    test.compare(
        size_on_disk, 0, f"Size of the placeholder on the disk is: '{size_on_disk}'"
    )


@Then('the file "|any|" should be downloaded')
def step(context, file_name):
    resource_path = get_resource_path(file_name)
    size_on_disk = get_file_size_on_disk(resource_path)
    file_size = get_file_size(resource_path)
    test.compare(
        size_on_disk,
        file_size,
        f"Original file size '{file_size}' is not equal to its size on disk '{size_on_disk}'",
    )



import re
import squish
from playwright.sync_api import sync_playwright, expect

from helpers.ConfigHelper import is_owncloud_client

envs = {}


def get_clipboard_text():
    try:
        return squish.getClipboardText()
    except:
        # Retry after 2 seconds
        squish.snooze(2)
        return squish.getClipboardText()


def authorize_via_webui(username, password, login_type='oidc'):
    global envs
    envs = {
        'OC_USERNAME': username.strip('"'),
        'OC_PASSWORD': password.strip('"'),
        'OC_AUTH_URL': get_clipboard_text(),
    }

    with sync_playwright() as p:
        browser = p.chromium.launch()
        context = browser.new_context(ignore_https_errors=True)
        page = context.new_page()
        page.goto(envs['OC_AUTH_URL'])
        if login_type == 'oidc':
            oidc_login(page)
        else:
            oauth_login(page)
        context.close()
        browser.close()


def oidc_login(page):
    if not is_owncloud_client():
        page.fill('#email', envs['OC_USERNAME'])
        page.click('span:text-is("Next")')
        page.fill('#password', envs['OC_PASSWORD'])
        page.click('span:text-is("Sign in")')
        page.wait_for_selector('text=Login Successful')
    else:
        # login
        page.fill('#oc-login-username', envs['OC_USERNAME'])
        page.fill('#oc-login-password', envs['OC_PASSWORD'])
        page.click('button[type=submit]')
        # allow permissions
        page.click('button >> text=Allow')
        # confirm successful login
        page.wait_for_selector('text=Login Successful')


def oauth_login(page):
    # login
    page.fill('#user', envs['OC_USERNAME'])
    page.fill('#password', envs['OC_PASSWORD'])
    page.click('button[type=submit]')
    # authorize
    page.click('button >> text=Authorize')
    # confirm successful login
    expect(page.locator('span.error')).to_have_text(
        re.compile('The application was authorized successfully')
    )



from os import path
import names
import squish

from helpers.SetupClientHelper import (
    get_current_user_sync_path,
    set_current_user_sync_path,
)
from helpers.ConfigHelper import get_config


class SyncConnectionWizard:
    ADD_FOLDER_SYNC_CONNECTION_WIZARD = {
        "name": "FolderWizardSourcePage",
        "type": "OCC::FolderWizardLocalPath",
        "visible": 1,
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
    }
    CHOOSE_LOCAL_SYNC_FOLDER = {
        "name": "localFolderLineEdit",
        "type": "QLineEdit",
        "visible": 1,
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
    }
    BACK_BUTTON = {
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
        "type": "QPushButton",
        "text": "< &Back",
        "visible": 1,
    }
    NEXT_BUTTON = {
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
        "type": "QPushButton",
        "text": "&Next >",
        "visible": 1,
    }
    SELECTIVE_SYNC_ROOT_FOLDER = {
        "column": 0,
        "container": names.folder_Sync_Connection_Deselect_remote_folders_QTreeWidget,
        "text": get_config("client_name"),
        "type": "QModelIndex",
    }
    SYNC_DIALOG_FOLDER_TREE = {
        "column": 0,
        "container": names.deselect_remote_folders_you_do_not_wish_to_synchronize_ownCloud_QModelIndex,
        "type": "QModelIndex",
    }
    ADD_SYNC_CONNECTION_BUTTON = {
        "name": "qt_wizard_finish",
        "type": "QPushButton",
        "visible": 1,
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
    }
    REMOTE_FOLDER_TREE = {
        "container": names.add_Folder_Sync_Connection_groupBox_QGroupBox,
        "name": "folderTreeWidget",
        "type": "QTreeWidget",
        "visible": 1,
    }
    VFS_CHECKBOX = {
        "text": "Use virtual files instead of downloading content immediately",
        "type": "QCheckBox",
        "unnamed": 1,
        "visible": 1,
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
    }
    SELECTIVE_SYNC_TREE_HEADER = {
        "container": names.folder_Sync_Connection_Deselect_remote_folders_QTreeWidget,
        "orientation": 1,
        "type": "QHeaderView",
        "unnamed": 1,
        "visible": 1,
    }
    CANCEL_FOLDER_SYNC_CONNECTION_WIZARD = {
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
        "name": "qt_wizard_cancel",
        "type": "QPushButton",
        "visible": 1,
    }
    SPACE_NAME_SELECTOR = {
        "container": names.quickWidget_scrollView_ScrollView,
        "type": "Label",
        "visible": True,
    }
    CREATE_REMOTE_FOLDER_BUTTON = {
        "container": names.add_Folder_Sync_Connection_groupBox_QGroupBox,
        "name": "addFolderButton",
        "type": "QPushButton",
        "visible": 1,
    }
    CREATE_REMOTE_FOLDER_INPUT = {
        "buddy": names.create_Remote_Folder_Enter_the_name_of_the_new_folder_to_be_created_below_QLabel,
        "type": "QLineEdit",
        "unnamed": 1,
        "visible": 1,
    }
    CREATE_REMOTE_FOLDER_CONFIRM_BUTTON = {
        "text": "OK",
        "type": "QPushButton",
        "unnamed": 1,
        "visible": 1,
        "window": names.create_Remote_Folder_QInputDialog,
    }
    REFRESH_BUTTON = {
        "container": names.add_Folder_Sync_Connection_groupBox_QGroupBox,
        "name": "refreshButton",
        "type": "QPushButton",
        "visible": 1,
    }
    REMOTE_FOLDER_SELECTION_INPUT = {
        "name": "folderEntry",
        "type": "QLineEdit",
        "visible": 1,
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
    }
    ADD_FOLDER_SYNC_BUTTON = {
        "checkable": False,
        "container": names.stackedWidget_quickWidget_OCC_QmlUtils_OCQuickWidget,
        "id": "addSyncButton",
        "type": "Button",
        "unnamed": 1,
        "visible": True,
    }
    WARN_LABEL = {
        "window": names.add_Folder_Sync_Connection_OCC_FolderWizard,
        "name": "warnLabel",
        "type": "QLabel",
        "visible": 1,
    }

    @staticmethod
    def set_sync_path_oc10(sync_path=""):
        squish.waitForObject(SyncConnectionWizard.ADD_FOLDER_SYNC_CONNECTION_WIZARD)
        squish.waitForObject(SyncConnectionWizard.CHOOSE_LOCAL_SYNC_FOLDER).setText("")
        if sync_path:
            squish.type(SyncConnectionWizard.CHOOSE_LOCAL_SYNC_FOLDER, sync_path)
            set_current_user_sync_path(sync_path)
        else:
            squish.type(
                SyncConnectionWizard.CHOOSE_LOCAL_SYNC_FOLDER,
                get_current_user_sync_path(),
            )
        SyncConnectionWizard.next_step()

    @staticmethod
    def set_sync_path_ocis(sync_path):
        if not sync_path:
            sync_path = path.join(
                get_current_user_sync_path(), get_config("syncConnectionName")
            )
        squish.type(
            squish.waitForObject(SyncConnectionWizard.CHOOSE_LOCAL_SYNC_FOLDER),
            "<Ctrl+A>",
        )
        squish.type(
            SyncConnectionWizard.CHOOSE_LOCAL_SYNC_FOLDER,
            sync_path,
        )
        SyncConnectionWizard.next_step()

    @staticmethod
    def set_sync_path(sync_path=""):
        if get_config("ocis"):
            SyncConnectionWizard.set_sync_path_ocis(sync_path)
        else:
            SyncConnectionWizard.set_sync_path_oc10(sync_path)

    @staticmethod
    def next_step():
        squish.clickButton(squish.waitForObject(SyncConnectionWizard.NEXT_BUTTON))

    @staticmethod
    def back():
        squish.clickButton(squish.waitForObject(SyncConnectionWizard.BACK_BUTTON))

    @staticmethod
    def select_remote_destination_folder(folder):
        squish.mouseClick(
            squish.waitForObjectItem(SyncConnectionWizard.REMOTE_FOLDER_TREE, folder)
        )
        SyncConnectionWizard.next_step()

    @staticmethod
    def deselect_all_remote_folders():
        # NOTE: checkbox does not have separate object
        # click on (11,11) which is a checkbox
        squish.mouseClick(
            squish.waitForObject(SyncConnectionWizard.SELECTIVE_SYNC_ROOT_FOLDER),
            11,
            11,
            squish.Qt.NoModifier,
            squish.Qt.LeftButton,
        )

    @staticmethod
    def enable_disable_vfs_support(action="disable"):
        if action not in ["enable", "disable"]:
            raise ValueError("Invalid action: " + action)

        checked = squish.waitForObjectExists(SyncConnectionWizard.VFS_CHECKBOX).checked
        if (action == "enable") == checked:
            return
        squish.clickButton(squish.waitForObject(SyncConnectionWizard.VFS_CHECKBOX))

    @staticmethod
    def select_folders_to_sync(folders):
        # first deselect all
        SyncConnectionWizard.deselect_all_remote_folders()
        for folder in folders:
            folder_levels = folder.strip("/").split("/")
            parent_selector = None
            for sub_folder in folder_levels:
                if not parent_selector:
                    SyncConnectionWizard.SYNC_DIALOG_FOLDER_TREE["text"] = sub_folder
                    parent_selector = SyncConnectionWizard.SYNC_DIALOG_FOLDER_TREE
                    selector = parent_selector
                else:
                    selector = {
                        "column": "0",
                        "container": parent_selector,
                        "text": sub_folder,
                        "type": "QModelIndex",
                    }
                if (
                    len(folder_levels) == 1
                    or folder_levels.index(sub_folder) == len(folder_levels) - 1
                ):
                    # NOTE: checkbox does not have separate object
                    # click on (11,11) which is a checkbox to unselect the folder
                    squish.mouseClick(
                        squish.waitForObject(selector),
                        11,
                        11,
                        squish.Qt.NoModifier,
                        squish.Qt.LeftButton,
                    )
                else:
                    squish.doubleClick(squish.waitForObject(selector))

    @staticmethod
    def sort_by(header_text):
        squish.mouseClick(
            squish.waitForObject(
                {
                    "container": SyncConnectionWizard.SELECTIVE_SYNC_TREE_HEADER,
                    "text": header_text,
                    "type": "HeaderViewItem",
                    "visible": True,
                }
            )
        )

    @staticmethod
    def add_sync_connection():
        squish.clickButton(
            squish.waitForObject(SyncConnectionWizard.ADD_SYNC_CONNECTION_BUTTON)
        )

    @staticmethod
    def get_item_name_from_row(row_index):
        folder_row = {
            "row": row_index,
            "container": SyncConnectionWizard.SELECTIVE_SYNC_ROOT_FOLDER,
            "type": "QModelIndex",
        }
        return str(squish.waitForObjectExists(folder_row).displayText)

    @staticmethod
    def is_root_folder_checked():
        state = squish.waitForObject(SyncConnectionWizard.SELECTIVE_SYNC_ROOT_FOLDER)[
            "checkState"
        ]
        return state == "checked"

    @staticmethod
    def cancel_folder_sync_connection_wizard():
        squish.clickButton(
            squish.waitForObject(
                SyncConnectionWizard.CANCEL_FOLDER_SYNC_CONNECTION_WIZARD
            )
        )

    @staticmethod
    def select_space(space_name):
        selector = SyncConnectionWizard.SPACE_NAME_SELECTOR.copy()
        selector["text"] = space_name
        squish.mouseClick(squish.waitForObject(selector))

    @staticmethod
    def sync_space(space_name):
        SyncConnectionWizard.select_space(space_name)
        SyncConnectionWizard.next_step()
        SyncConnectionWizard.set_sync_path(
            path.join(get_current_user_sync_path(), space_name)
        )
        SyncConnectionWizard.add_sync_connection()

    @staticmethod
    def create_folder_in_remote_destination(folder_name):
        squish.clickButton(
            squish.waitForObject(SyncConnectionWizard.CREATE_REMOTE_FOLDER_BUTTON)
        )
        squish.type(
            squish.waitForObject(SyncConnectionWizard.CREATE_REMOTE_FOLDER_INPUT),
            folder_name,
        )
        squish.clickButton(
            squish.waitForObject(
                SyncConnectionWizard.CREATE_REMOTE_FOLDER_CONFIRM_BUTTON
            )
        )

    @staticmethod
    def refresh_remote():
        squish.clickButton(squish.waitForObject(SyncConnectionWizard.REFRESH_BUTTON))

    @staticmethod
    def generate_remote_folder_selector(folder_name, parent_container=None):
        if not parent_container:
            parent_container = {
                "container": names.groupBox_folderTreeWidget_QTreeWidget,
                "text": "ownCloud",
                "type": "QModelIndex",
            }
        return {
            "container": parent_container,
            "text": folder_name,
            "type": "QModelIndex",
        }

    @staticmethod
    def has_remote_folder(folder_name):
        folder_tree = folder_name.strip("/").split("/")
        parent_container = None

        for folder in folder_tree:
            folder_selector = SyncConnectionWizard.generate_remote_folder_selector(
                folder, parent_container
            )
            try:
                if parent_container:
                    squish.doubleClick(parent_container)

                squish.waitForObject(folder_selector)

                parent_container = folder_selector
            except:
                return False, None
        return True, parent_container

    @staticmethod
    def is_remote_folder_selected(folder_selector):
        return squish.waitForObjectExists(folder_selector).selected

    @staticmethod
    def open_sync_connection_wizard():
        squish.mouseClick(
            squish.waitForObject(SyncConnectionWizard.ADD_FOLDER_SYNC_BUTTON)
        )

    @staticmethod
    def get_local_sync_path():
        return str(
            squish.waitForObjectExists(
                SyncConnectionWizard.CHOOSE_LOCAL_SYNC_FOLDER
            ).displayText
        )

    @staticmethod
    def get_warn_label():
        return str(squish.waitForObjectExists(SyncConnectionWizard.WARN_LABEL).text)

    @staticmethod
    def is_add_sync_folder_button_enabled():
        return squish.waitForObjectExists(
            SyncConnectionWizard.ADD_FOLDER_SYNC_BUTTON
        ).enabled

    @staticmethod
    def is_space_available(space_name):
        selector = SyncConnectionWizard.SPACE_NAME_SELECTOR.copy()
        selector["text"] = space_name
        space_available = False
        try:
            squish.waitForObject(selector)
            space_available = True
        except:
            pass
        return space_available



import test
import names
import squish

from pageObjects.EnterPassword import EnterPassword

from helpers.WebUIHelper import authorize_via_webui
from helpers.ConfigHelper import get_config
from helpers.SetupClientHelper import (
    create_user_sync_path,
    get_temp_resource_path,
    set_current_user_sync_path,
)
from helpers.SyncHelper import listen_sync_status_for_item


class AccountConnectionWizard:
    SERVER_ADDRESS_BOX = {
        "container": names.setupWizardWindow_contentWidget_QStackedWidget,
        "name": "urlLineEdit",
        "type": "QLineEdit",
        "visible": 1,
    }
    NEXT_BUTTON = {
        "container": names.settings_dialogStack_QStackedWidget,
        "name": "nextButton",
        "type": "QPushButton",
        "visible": 1,
    }
    CONFIRM_INSECURE_CONNECTION_BUTTON = {
        "text": "Confirm",
        "type": "QPushButton",
        "unnamed": 1,
        "visible": 1,
        "window": names.insecure_connection_QMessageBox,
    }
    USERNAME_BOX = {
        "container": names.contentWidget_OCC_QmlUtils_OCQuickWidget,
        "id": "userNameField",
        "type": "TextField",
        "visible": True,
    }
    PASSWORD_BOX = {
        "container": names.contentWidget_OCC_QmlUtils_OCQuickWidget,
        "id": "passwordField",
        "type": "TextField",
        "visible": True,
    }
    SELECT_LOCAL_FOLDER = {
        "container": names.advancedConfigGroupBox_localDirectoryGroupBox_QGroupBox,
        "name": "localDirectoryLineEdit",
        "type": "QLineEdit",
        "visible": 1,
    }
    DIRECTORY_NAME_BOX = {
        "container": names.advancedConfigGroupBox_localDirectoryGroupBox_QGroupBox,
        "name": "chooseLocalDirectoryButton",
        "type": "QToolButton",
        "visible": 1,
    }
    CHOOSE_BUTTON = {
        "text": "Choose",
        "type": "QPushButton",
        "unnamed": 1,
        "visible": 1,
        "window": names.qFileDialog_QFileDialog,
    }
    ERROR_LABEL = {
        "name": "errorMessageLabel",
        "type": "QLabel",
        "visible": 1,
        "window": names.setupWizardWindow_OCC_Wizard_SetupWizardWindow,
    }
    BASIC_CREDENTIAL_PAGE = {
        "container": names.contentWidget_contentWidget_QStackedWidget,
        "type": "OCC::Wizard::BasicCredentialsSetupWizardPage",
        "visible": 1,
    }
    OAUTH_CREDENTIAL_PAGE = {
        "container": names.contentWidget_contentWidget_QStackedWidget,
        "type": "OCC::Wizard::OAuthCredentialsSetupWizardPage",
        "visible": 1,
    }
    COPY_URL_TO_CLIPBOARD_BUTTON = {
        "container": names.contentWidget_OCC_QmlUtils_OCQuickWidget,
        "id": "copyToClipboardButton",
        "type": "Button",
        "visible": True,
    }
    CONF_SYNC_MANUALLY_RADIO_BUTTON = {
        "container": names.advancedConfigGroupBox_syncModeGroupBox_QGroupBox,
        "name": "configureSyncManuallyRadioButton",
        "type": "QRadioButton",
        "visible": 1,
    }
    ADVANCED_CONFIGURATION_CHECKBOX = {
        "container": names.setupWizardWindow_contentWidget_QStackedWidget,
        "name": "advancedConfigGroupBox",
        "type": "QGroupBox",
        "visible": 1,
    }
    DIRECTORY_NAME_EDIT_BOX = {
        "buddy": names.qFileDialog_fileNameLabel_QLabel,
        "name": "fileNameEdit",
        "type": "QLineEdit",
        "visible": 1,
    }
    VIRTUAL_FILE_RADIO_BUTTON = {
        "container": names.advancedConfigGroupBox_syncModeGroupBox_QGroupBox,
        "name": "useVfsRadioButton",
        "type": "QRadioButton",
        "visible": 1,
    }
    SYNC_EVERYTHING_RADIO_BUTTON = {
        "container": names.advancedConfigGroupBox_syncModeGroupBox_QGroupBox,
        "name": "syncEverythingRadioButton",
        "type": "QRadioButton",
        "visible": 1,
    }

    @staticmethod
    def add_server(server_url):
        squish.mouseClick(
            squish.waitForObject(AccountConnectionWizard.SERVER_ADDRESS_BOX)
        )
        squish.type(
            squish.waitForObject(AccountConnectionWizard.SERVER_ADDRESS_BOX),
            server_url,
        )
        AccountConnectionWizard.next_step()

        if not get_config("ocis"):
            try:
                squish.clickButton(
                    squish.waitForObject(
                        AccountConnectionWizard.CONFIRM_INSECURE_CONNECTION_BUTTON, 1000
                    )
                )
            except:
                test.log("No insecure connection warning for server " + server_url)

    @staticmethod
    def accept_certificate():
        squish.clickButton(squish.waitForObject(EnterPassword.ACCEPT_CERTIFICATE_YES))

    @staticmethod
    def add_user_credentials(username, password, oauth=False):
        if get_config("ocis"):
            AccountConnectionWizard.oidc_login(username, password)
        elif oauth:
            AccountConnectionWizard.oauth_login(username, password)
        else:
            AccountConnectionWizard.basic_login(username, password)

    @staticmethod
    def basic_login(username, password):
        squish.mouseClick(squish.waitForObject(AccountConnectionWizard.USERNAME_BOX))
        squish.nativeType(username)
        squish.mouseClick(squish.waitForObject(AccountConnectionWizard.PASSWORD_BOX))
        squish.nativeType(password)
        AccountConnectionWizard.next_step()

    @staticmethod
    def oidc_login(username, password):
        AccountConnectionWizard.browser_login(username, password, "oidc")

    @staticmethod
    def oauth_login(username, password):
        AccountConnectionWizard.browser_login(username, password, "oauth")

    @staticmethod
    def browser_login(username, password, login_type=None):
        # wait 500ms for copy button to fully load
        squish.snooze(1 / 2)
        squish.mouseClick(
            squish.waitForObject(AccountConnectionWizard.COPY_URL_TO_CLIPBOARD_BUTTON)
        )
        authorize_via_webui(username, password, login_type)

    @staticmethod
    def next_step():
        squish.clickButton(
            squish.waitForObjectExists(AccountConnectionWizard.NEXT_BUTTON)
        )

    @staticmethod
    def select_sync_folder(user):
        # create sync folder for user
        sync_path = create_user_sync_path(user)

        AccountConnectionWizard.select_advanced_config()
        squish.mouseClick(
            squish.waitForObject(AccountConnectionWizard.DIRECTORY_NAME_BOX)
        )
        squish.type(
            squish.waitForObject(AccountConnectionWizard.DIRECTORY_NAME_EDIT_BOX),
            sync_path,
        )
        squish.clickButton(squish.waitForObject(AccountConnectionWizard.CHOOSE_BUTTON))
        return sync_path

    @staticmethod
    def set_temp_folder_as_sync_folder(folder_name):
        sync_path = get_temp_resource_path(folder_name)

        # clear the current path
        squish.mouseClick(
            squish.waitForObject(AccountConnectionWizard.SELECT_LOCAL_FOLDER)
        )

        squish.waitForObject(AccountConnectionWizard.SELECT_LOCAL_FOLDER).setText("")

        squish.type(
            squish.waitForObject(AccountConnectionWizard.SELECT_LOCAL_FOLDER),
            sync_path,
        )
        set_current_user_sync_path(sync_path)
        return sync_path

    @staticmethod
    def add_account(account_details):
        AccountConnectionWizard.add_account_information(account_details)
        AccountConnectionWizard.next_step()

    @staticmethod
    def add_account_information(account_details):
        if account_details["server"]:
            AccountConnectionWizard.add_server(account_details["server"])
            if get_config("ocis"):
                AccountConnectionWizard.accept_certificate()
        if account_details["user"]:
            AccountConnectionWizard.add_user_credentials(
                account_details["user"],
                account_details["password"],
                account_details["oauth"],
            )
        sync_path = ""
        if account_details["sync_folder"]:
            AccountConnectionWizard.select_advanced_config()
            sync_path = AccountConnectionWizard.set_temp_folder_as_sync_folder(
                account_details["sync_folder"]
            )
        elif account_details["user"]:
            sync_path = AccountConnectionWizard.select_sync_folder(
                account_details["user"]
            )
        if sync_path:
            # listen for sync status
            listen_sync_status_for_item(sync_path)

    @staticmethod
    def select_manual_sync_folder_option():
        squish.clickButton(
            squish.waitForObject(
                AccountConnectionWizard.CONF_SYNC_MANUALLY_RADIO_BUTTON
            )
        )

    @staticmethod
    def select_vfs_option():
        squish.clickButton(
            squish.waitForObject(AccountConnectionWizard.VIRTUAL_FILE_RADIO_BUTTON)
        )

    @staticmethod
    def select_download_everything_option():
        squish.clickButton(
            squish.waitForObject(AccountConnectionWizard.SYNC_EVERYTHING_RADIO_BUTTON)
        )

    @staticmethod
    def get_error_message():
        return str(squish.waitForObjectExists(AccountConnectionWizard.ERROR_LABEL).text)

    @staticmethod
    def is_new_connection_window_visible():
        visible = False
        try:
            squish.waitForObject(AccountConnectionWizard.SERVER_ADDRESS_BOX)
            visible = True
        except:
            pass
        return visible

    @staticmethod
    def is_credential_window_visible():
        visible = False
        try:
            if get_config("ocis"):
                squish.waitForObject(AccountConnectionWizard.OAUTH_CREDENTIAL_PAGE)
            else:
                squish.waitForObject(AccountConnectionWizard.BASIC_CREDENTIAL_PAGE)
            visible = True
        except:
            pass
        return visible

    @staticmethod
    def select_advanced_config():
        squish.waitForObject(
            AccountConnectionWizard.ADVANCED_CONFIGURATION_CHECKBOX
        ).setChecked(True)

    @staticmethod
    def can_change_local_sync_dir():
        can_change = False
        try:
            squish.waitForObjectExists(AccountConnectionWizard.SELECT_LOCAL_FOLDER)
            squish.clickButton(
                squish.waitForObject(AccountConnectionWizard.DIRECTORY_NAME_BOX)
            )
            squish.waitForObjectExists(AccountConnectionWizard.CHOOSE_BUTTON)
            can_change = True
        except:
            pass
        return can_change

    @staticmethod
    def is_sync_everything_option_checked():
        return squish.waitForObjectExists(
            AccountConnectionWizard.SYNC_EVERYTHING_RADIO_BUTTON
        ).checked

    @staticmethod
    def is_vfs_option_checked():
        return squish.waitForObjectExists(
            AccountConnectionWizard.VIRTUAL_FILE_RADIO_BUTTON
        ).checked

    @staticmethod
    def get_local_sync_path():
        return str(
            squish.waitForObjectExists(
                AccountConnectionWizard.SELECT_LOCAL_FOLDER
            ).displayText
        )



import json

import helpers.api.http_helper as request
from helpers.api.utils import url_join
from helpers.ConfigHelper import get_config


# store the apps initial state to restore it after the test
# E.g.:
#   "app_name": "enabled"
apps_initial_state = {}


def format_json(url):
    return url + "?format=json"


def get_ocs_url():
    return url_join(get_config("localBackendUrl"), "ocs", "v2.php")


def get_provisioning_url(*paths):
    return format_json(url_join(get_ocs_url(), "cloud", *paths))


def check_success_ocs_status(response):
    if response.text:
        ocs_data = json.loads(response.text)
        if ocs_data["ocs"]["meta"]["statuscode"] not in [100, 200]:
            raise AssertionError("Request failed." + response.text)
    else:
        raise ValueError(
            "No OCS response body. HTTP status was " + str(response.status_code)
        )


def create_user(username, password, displayname, email):
    url = get_provisioning_url("users")
    body = {
        "userid": username,
        "password": password,
        "displayname": displayname,
        "email": email,
    }
    response = request.post(url, body)
    request.assert_http_status(response, 200, f"Failed to create user '{username}'")
    check_success_ocs_status(response)

    # oc10 does not set display name while creating user,
    # so we need update the user info
    user_url = get_provisioning_url("users", username)
    display_name_body = {"key": "displayname", "value": displayname}
    display_name_response = request.put(user_url, display_name_body)
    request.assert_http_status(
        display_name_response, 200, f"Failed to update displayname of user '{username}'"
    )
    check_success_ocs_status(display_name_response)

    return {
        "id": username,
        "username": username,
        "password": password,
        "displayname": displayname,
        "email": email,
    }


def delete_user(user_id):
    url = get_provisioning_url("users", user_id)
    response = request.delete(url)
    request.assert_http_status(response, 200, f"Failed to delete user '{user_id}'")
    check_success_ocs_status(response)


def create_group(group_name):
    body = {"groupid": group_name}
    response = request.post(get_provisioning_url("groups"), body)
    request.assert_http_status(response, 200, f"Failed to create group '{group_name}'")
    check_success_ocs_status(response)
    return {"id": group_name}


def delete_group(group_id):
    url = get_provisioning_url("groups", group_id)
    response = request.delete(url)
    request.assert_http_status(response, 200, f"Failed to delete group '{group_id}'")
    check_success_ocs_status(response)


def add_user_to_group(user, group_name):
    url = get_provisioning_url("users", user, "groups")
    body = {"groupid": group_name}
    response = request.post(url, body)
    request.assert_http_status(
        response, 200, f"Failed to add user '{user}' to group '{group_name}'"
    )
    check_success_ocs_status(response)


def get_enabled_apps():
    url = get_provisioning_url("apps")
    response = request.get(f"{url}&filter=enabled")
    request.assert_http_status(response, 200, "Failed to get enabled apps")
    check_success_ocs_status(response)
    return json.loads(response.text)["ocs"]["data"]["apps"]


def enable_app(app_name):
    url = get_provisioning_url("apps", app_name)
    response = request.post(url)
    request.assert_http_status(response, 200, f"Failed to enable app '{app_name}'")
    check_success_ocs_status(response)


def disable_app(app_name):
    url = get_provisioning_url("apps", app_name)
    response = request.delete(url)
    request.assert_http_status(response, 200, f"Failed to disable app '{app_name}'")
    check_success_ocs_status(response)


def setup_app(app_name, action):
    if action.startswith("enable"):
        enable_app(app_name)
    elif action.startswith("disable"):
        disable_app(app_name)
    else:
        raise ValueError("Unknown action: " + action)

    if app_name not in apps_initial_state:
        enabled_apps = get_enabled_apps()
        if app_name in enabled_apps:
            apps_initial_state[app_name] = "enabled"
        apps_initial_state[app_name] = "disabled"


def restore_apps_state():
    for app_name, action in apps_initial_state.items():
        setup_app(app_name, action)
    apps_initial_state.clear()



import os
import json

import helpers.api.http_helper as request
from helpers.api.utils import url_join
from helpers.ConfigHelper import get_config
from helpers.FilesHelper import write_file
from helpers.UserHelper import get_username_for_user
from helpers.api.provisioning import created_users
from helpers.FilesHelper import get_file_for_upload

tokens = {}
personal_sync_folder_id = {}
created_spaces = {}
available_roles = {
    'viewer': 6,
    'downloader': 2,
    'uploader': 7,
    'collaborator': 3,
    'manager': 4,
}


def get_headers(user, extra_headers=None):
    token = generate_xsrf_token(user)
    headers = {
        'x-accellion-version': '28',
        'x-xsrf-token': token['x_xsrf_token'],
        'cookie': f'ACC-XSRF-TOKEN={token["acc_xsrf_token"]}; web_token={token["web_token"]}',
    }
    if extra_headers:
        headers.update(extra_headers)
    return headers


def generate_xsrf_token(user):
    if user in tokens:
        return tokens[user]
    url = url_join(get_config('localBackendUrl'), 'rest/users/actions/login')
    headers = {'content-type': 'application/json', 'x-accellion-version': '28'}
    for created_user in created_users.values():
        if created_user['username'] == user:
            password = created_user['password']
            break
    body = json.dumps({'username': user, 'password': password})
    response = request.send_request(url, 'POST', body, headers)
    assert response.status_code == 200, 'Failed to get the token'

    acc_xsrf_token = None
    web_token = None
    x_xsrf_token = None
    for cookie in response.cookies:
        if cookie.name == 'ACC-XSRF-TOKEN':
            acc_xsrf_token = cookie.value
            x_xsrf_token = cookie.value
        elif cookie.name == 'web_token':
            web_token = cookie.value
    tokens[user] = {
        'acc_xsrf_token': acc_xsrf_token,
        'web_token': web_token,
        'x_xsrf_token': x_xsrf_token,
    }
    return tokens[user]


def resource_exists(user, resource, space=None):
    user = get_username_for_user(user)
    resource_name = os.path.basename(resource)
    folder_path = os.path.dirname(resource)
    if space:
        folder_id = get_space_id(user, space)
    else:
        folder_id = get_folder_id_by_path(user, folder_path)
    return get_resource(user, resource_name, folder_id) is not None


def get_file_content(user, resource, space=None):
    user = get_username_for_user(user)
    r = get_resource(user, resource, space=space)
    url = url_join(get_config('localBackendUrl'), f'rest/files/{r["id"]}/content')
    headers = get_headers(user)
    response = request.get(url, headers=headers)
    assert response.status_code == 200, f'Could not get content of file: {resource}'
    return response.text


def create_folder(user, folder_name):
    user = get_username_for_user(user)
    folder_to_create = os.path.basename(folder_name)
    folder_path = os.path.dirname(folder_name)
    folder_id = get_folder_id_by_path(user, folder_path)
    url = url_join(
        get_config('localBackendUrl'),
        f'rest/folders/{folder_id}/folders',
    )
    headers = get_headers(user, {'content-type': 'application/json'})
    body = json.dumps(
        {
            'description': '',
            'fileLifetime': '0',
            'name': folder_to_create,
            'syncable': '1',
        }
    )
    response = request.post(url, body, headers)
    assert (
        response.status_code == 201
    ), f'Could not create the folder: {folder_name} for user {user}'


def get_folder_id_by_path(user, folder_path=None):
    parts = folder_path.split(os.sep)
    current_folder_id = get_personal_sync_folder_id(user)
    if not folder_path:
        return current_folder_id

    for part in parts:
        if folder := get_resource(user, part, current_folder_id, 'd'):
            current_folder_id = folder['id']
    return current_folder_id


def upload_file(user, file_name, file_content=None):
    user = get_username_for_user(user)
    filename = os.path.basename(file_name)
    if file_content:
        # create file in temp folder before uploading
        file_path = f'{get_config("tempFolderPath")}/{filename}'
        write_file(file_path, file_content)
    else:
        file_path = get_file_for_upload(filename)
    file_size = os.path.getsize(file_path)
    folder_path = os.path.dirname(file_name)
    folder_id = get_folder_id_by_path(user, folder_path)

    # initiate upload
    upload_uri = initiate_upload(user, filename, file_size, folder_id)

    # finalize upload
    finalize_upload(user, filename, file_path, file_size, upload_uri)


def initiate_upload(user, file_name, file_size, folder_id):
    url = url_join(
        get_config('localBackendUrl'),
        f'rest/folders/{folder_id}/actions/initiateUpload',
    )
    headers = get_headers(user, {'content-type': 'application/json'})
    json_body = json.dumps(
        {'filename': file_name, 'totalSize': file_size, 'totalChunks': 1}
    )

    response = request.post(url, body=json_body, headers=headers)
    assert (
        response.status_code == 201
    ), f'Failed to initiate upload for file {file_name}.'

    data = json.loads(response.text)
    return data['uri']


def finalize_upload(user, file_name, file_path, file_size, upload_uri):
    url = url_join(get_config('localBackendUrl'), upload_uri)
    headers = get_headers(user)
    with open(file_path, 'rb') as f:
        files = {'content': (file_name, f, 'application/octet-stream')}
        body = {
            'compressionMode': 'NORMAL',
            'compressionSize': file_size,
            'originalSize': file_size,
            'index': 0,
        }
        response = request.post(url, body=body, headers=headers, files=files)
        assert response.status_code == 201, f'Failed to upload file {file_name}.'


def delete_resource(user, resource):
    user = get_username_for_user(user)
    r = get_resource(user, resource)
    if r['type'] == 'd':
        uri = 'folders'
    else:
        uri = 'files'
    url = url_join(
        get_config('localBackendUrl'),
        f'rest/{uri}?partialSuccess=true&id:in={r["id"]}',
    )
    headers = get_headers(user)
    response = request.delete(url, headers)
    assert response.status_code == 204, f'Cound not delete resource: {resource}'


def get_personal_sync_folder_id(user):
    if user in personal_sync_folder_id:
        return personal_sync_folder_id[user]

    url = url_join(get_config('localBackendUrl'), 'rest/folders/top')
    headers = get_headers(user)
    response = request.get(url, headers)
    assert (
        response.status_code == 200
    ), f'Failed to get the personal sync folder id for user {user}'
    data = json.loads(response.text)

    for folder in data['data']:
        if folder['name'] == get_config('personal_sync_folder'):
            personal_sync_folder_id[user] = folder['id']
            return personal_sync_folder_id[user]
    raise ValueError(f'Cound not find personal sync folder id for user {user}')


# return resources inside sync folder
# status=False returns resources that are not deleted
# status=True returns resources that are deleted and are stored in deleted content
def get_resources_inside_folder(user, status=False, folder_id=None):
    if not folder_id:
        folder_id = get_personal_sync_folder_id(user)
    url = url_join(
        get_config('localBackendUrl'),
        f'rest/folders/{folder_id}/children?deleted={status}',
    )
    headers = get_headers(user)
    response = request.get(url, headers)
    assert response.status_code == 200, 'Failed to get the resources inside folder'

    data = json.loads(response.text)
    return data['data']


def get_resource_ids(user, status=False):
    resources = get_resources_inside_folder(user, status)
    folder_ids = []
    file_ids = []
    for resource in resources:
        if resource['type'] == 'd':
            folder_ids.append(resource['id'])
        if resource['type'] == 'f':
            file_ids.append(resource['id'])
    return folder_ids, file_ids


def delete_all_resources():
    for user in created_users.values():
        folder_ids, file_ids = get_resource_ids(user['username'])
        headers = get_headers(user['username'])
        # Delete all folders
        if len(folder_ids) > 0:
            url = url_join(
                get_config('localBackendUrl'),
                f'rest/folders?partialSuccess=true&id:in={",".join(folder_ids)}',
            )
            response = request.delete(url, headers)
            assert response.status_code == 204, 'Failed to delete all folders.'

        # Delete all files
        if len(file_ids) > 0:
            url = url_join(
                get_config('localBackendUrl'),
                f'rest/files?partialSuccess=true&id:in={",".join(file_ids)}',
            )
            response = request.delete(url, headers)
            assert response.status_code == 204, 'Failed to delete all files.'


def permanently_delete_all_resources():
    for user in created_users.values():
        folder_ids, file_ids = get_resource_ids(user['username'], True)
        headers = get_headers(user['username'])
        # Permanently delete all folders
        if len(folder_ids) > 0:
            url = url_join(
                get_config('localBackendUrl'),
                f'rest/folders/actions/permanent?partialSuccess=true&id:in={",".join(folder_ids)}',
            )
            response = request.delete(url, headers)
            assert (
                response.status_code == 204
            ), 'Failed to permanently delete all folders.'

        # Permanently delete all files
        if len(file_ids) > 0:
            url = url_join(
                get_config('localBackendUrl'),
                f'rest/files/actions/permanent?partialSuccess=true&id:in={",".join(file_ids)}',
            )
            response = request.delete(url, headers)
            assert (
                response.status_code == 204
            ), 'Failed to permanently delete all files.'


def get_folder_items_count(user, folder_name):
    user = get_username_for_user(user)
    folder_id = get_folder_id_by_path(user, folder_name)
    resources = get_resources_inside_folder(user, folder_id=folder_id)
    return str(len(resources))


def get_resource(user, name, folder_id=None, resource_type=None, space=None):
    if space:
        folder_id = get_space_id(user, space)
    resources = get_resources_inside_folder(user, folder_id=folder_id)
    for resource in resources:
        if resource['name'] == name and (
            resource_type is None or resource['type'] == resource_type
        ):
            return resource
    return None


def create_space(space_name, user='admin'):
    user = get_username_for_user(user)
    url = url_join(get_config('localBackendUrl'), 'rest/folders/0/folders')
    headers = get_headers(user, {'content-type': 'application/json'})
    body = json.dumps(
        {
            'name': space_name,
            'parent': '0',
            'description': '',
            'expire': '0',
            'fileLifetime': '0',
            'syncable': '1',
            'secure': '0',
        }
    )
    response = request.post(url, body, headers)
    assert (
        response.status_code == 201
    ), f'Could not create space {space_name} for user {user}'
    if user not in created_spaces:
        created_spaces[user] = []
    created_spaces[user].append(space_name)


def get_spaces(user, deleted=False):
    url = url_join(get_config('localBackendUrl'), f'rest/folders/top?deleted={deleted}')
    headers = get_headers(user)
    response = request.get(url, headers)
    assert response.status_code == 200, f'Failed to get spaces for user {user}'
    data = json.loads(response.text)
    return data['data']


def get_space_id(user, space_name):
    spaces = get_spaces(user)
    for space in spaces:
        if space['name'] == space_name:
            return space['id']
    raise ValueError(f'Could not get space id for space {space_name}')


def get_deleted_space_ids(user):
    spaces = get_spaces(user, True)
    deleted_space_ids = []
    for space in spaces:
        deleted_space_ids.append(space['id'])
    return deleted_space_ids


def delete_all_spaces():
    for user in created_users.values():
        if (user := user['username']) in created_spaces:
            for space in created_spaces[user]:
                space_id = get_space_id(user, space)
                url = url_join(
                    get_config('localBackendUrl'),
                    f'rest/folders?partialSuccess=true&id:in={space_id}',
                )
                headers = get_headers(user)
                response = request.delete(url, headers)
                assert response.status_code == 204, f'Could not delete space: {space}'


def permanently_delete_all_spaces():
    for user in created_users.values():
        if (user := user['username']) in created_spaces:
            deleted_space_ids = get_deleted_space_ids(user)
            headers = get_headers(user)
            url = url_join(
                get_config('localBackendUrl'),
                f'rest/folders/actions/permanent?partialSuccess=true&id:in={",".join(deleted_space_ids)}',
            )
            response = request.delete(url, headers)
            assert (
                response.status_code == 204
            ), 'Failed to permanently delete all spaces'
    created_spaces.clear()


def create_space_folder(space, folder_name, user='admin'):
    user = get_username_for_user(user)
    space_id = get_space_id(user, space)
    url = url_join(get_config('localBackendUrl'), f'rest/folders/{space_id}/folders')
    headers = get_headers(user, {'content-type': 'application/json'})
    body = json.dumps(
        {
            'description': '',
            'fileLifetime': '0',
            'name': folder_name,
            'syncable': '1',
        }
    )
    response = request.post(url, body, headers)
    assert (
        response.status_code == 201
    ), f'Could not create the folder: {folder_name} in space {space}'


def upload_space_file(space, file_name, file_content, user='admin'):
    user = get_username_for_user(user)
    file_path = f'{get_config("tempFolderPath")}/{file_name}'
    write_file(file_path, file_content)
    file_size = os.path.getsize(file_path)
    space_id = get_space_id(user, space)
    # initiate upload
    upload_uri = initiate_upload(user, file_name, file_size, space_id)
    # finalize upload
    finalize_upload(user, file_name, file_path, file_size, upload_uri)


def add_user_to_space(user, space_name, role, space_owner='admin'):
    space_owner = get_username_for_user(space_owner)
    user = get_username_for_user(user)
    role_id = available_roles[role]
    space_id = get_space_id(space_owner, space_name)
    url = url_join(
        get_config('localBackendUrl'),
        f'rest/folders/{space_id}/members?updateIfExists=true&partialSuccess=true',
    )
    headers = get_headers(space_owner, {'content-type': 'application/json'})
    body = json.dumps(
        {'roleId': role_id, 'notify': True, 'notifyFileAdded': False, 'emails': [user]}
    )
    response = request.post(url, body, headers)
    assert (
        response.status_code == 201
    ), f'Failed to add user {user} to space {space_name}'



from helpers.api import provisioning, sharing_helper, webdav_helper as webdav
import helpers.api.oc10 as oc
from helpers.ConfigHelper import is_owncloud_client
import helpers.api.external_api as ext

from pageObjects.Toolbar import Toolbar


@Given('app "|any|" has been "|any|" in the server')
def step(context, app_name, action):
    oc.setup_app(app_name, action)


@Then(
    r'^as "([^"].*)" (?:file|folder) "([^"].*)" should not exist in the server',
    regexp=True,
)
def step(context, user_name, resource_name):
    if not is_owncloud_client():
        resource_exists = ext.resource_exists(user_name, resource_name)
    else:
        resource_exists = webdav.resource_exists(user_name, resource_name)
    test.compare(
        resource_exists,
        False,
        f"Resource '{resource_name}' should not exist, but does",
    )


@Then(
    r'^as "([^"].*)" (?:file|folder) "([^"].*)" should exist in the server', regexp=True
)
def step(context, user_name, resource_name):
    if not is_owncloud_client():
        resource_exists = ext.resource_exists(user_name, resource_name)
    else:
        resource_exists = webdav.resource_exists(user_name, resource_name)
    test.compare(
        resource_exists,
        True,
        f"Resource '{resource_name}' should exist, but does not",
    )


@Then('as "|any|" the file "|any|" should have the content "|any|" in the server')
def step(context, user_name, file_name, content):
    if not is_owncloud_client():
        text_content = ext.get_file_content(user_name, file_name)
    else:
        text_content = webdav.get_file_content(user_name, file_name)
    test.compare(
        text_content,
        content,
        f"File '{file_name}' should have content '{content}' but found '{text_content}'",
    )


@Then(
    r'as user "([^"].*)" the (?:file|folder) "([^"].*)" should have a public link in the server',
    regexp=True,
)
def step(context, user_name, resource_name):
    has_link_share = sharing_helper.has_public_link_share(user_name, resource_name)
    test.compare(
        has_link_share,
        True,
        f"Resource '{resource_name}' does not have public link share",
    )


@Then(
    r'as user "([^"].*)" the (?:file|folder) "([^"].*)" should not have any public link in the server',
    regexp=True,
)
def step(context, user_name, resource_name):
    has_link_share = sharing_helper.has_public_link_share(user_name, resource_name)
    test.compare(
        has_link_share, False, f"Resource '{resource_name}' have public link share"
    )


@Then(
    # pylint: disable=line-too-long
    r'the public should be able to download the (?:file|folder) "([^"].*)" without password from the last created public link by "([^"].*)" in the server',
    regexp=True,
)
def step(context, resource_name, link_creator):
    downloaded = sharing_helper.download_last_public_link_resource(
        link_creator, resource_name
    )
    test.compare(downloaded, True, 'Could not download public share')


@Then(
    # pylint: disable=line-too-long
    r'the public should not be able to download the (?:file|folder) "([^"].*)" from the last created public link by "([^"].*)" in the server',
    regexp=True,
)
def step(context, resource_name, link_creator):
    downloaded = sharing_helper.download_last_public_link_resource(
        link_creator, resource_name
    )
    test.compare(downloaded, False, 'Could download public share')


@Then(
    # pylint: disable=line-too-long
    r'the public should be able to download the (?:file|folder) "([^"].*)" with password "([^"].*)" from the last created public link by "([^"].*)" in the server',
    regexp=True,
)
def step(context, resource_name, public_link_password, link_creator):
    downloaded = sharing_helper.download_last_public_link_resource(
        link_creator, resource_name, public_link_password
    )
    test.compare(downloaded, True, 'Could not download public share')


@Then(
    r'as user "([^"].*)" folder "([^"].*)" should contain "([^"].*)" items in the server',
    regexp=True,
)
def step(context, user_name, folder_name, items_number):
    if not is_owncloud_client():
        total_items = ext.get_folder_items_count(user_name, folder_name)
    else:
        total_items = webdav.get_folder_items_count(user_name, folder_name)
    test.compare(
        total_items, items_number, f'Folder should contain {items_number} items'
    )


@Given('user "|any|" has created folder "|any|" in the server')
def step(context, user, folder_name):
    if not is_owncloud_client():
        ext.create_folder(user, folder_name)
    else:
        webdav.create_folder(user, folder_name)


@Given('user "|any|" has uploaded file with content "|any|" to "|any|" in the server')
def step(context, user, file_content, file_name):
    if not is_owncloud_client():
        ext.upload_file(user, file_name, file_content)
    else:
        webdav.create_file(user, file_name, file_content)


@When('the user clicks on the settings tab')
def step(context):
    Toolbar.open_settings_tab()


@When('user "|any|" uploads file with content "|any|" to "|any|" in the server')
def step(context, user, file_content, file_name):
    if not is_owncloud_client():
        ext.upload_file(user, file_name, file_content)
    else:
        webdav.create_file(user, file_name, file_content)


@When('user "|any|" deletes the folder "|any|" in the server')
def step(context, user, folder_name):
    if not is_owncloud_client():
        ext.delete_resource(user, folder_name)
    else:
        webdav.delete_resource(user, folder_name)


@Given('group "|any|" has been created in the server')
def step(context, group_name):
    provisioning.create_group(group_name)


@Given('user "|any|" has been added to group "|any|" in the server')
def step(context, user, group_name):
    provisioning.add_user_to_group(user, group_name)


@Given('user "|any|" has been created in the server with default attributes')
def step(context, user):
    provisioning.create_user(user)


@Given(
    # pylint: disable=line-too-long
    r'user "([^"].*)" has shared (?:file|folder) "([^"].*)" in the server with (user|group) "([^"].*)" with "([^"].*)" permission(?:s)?',
    regexp=True,
)
def step(context, user, resource, receiver_type, receiver, permissions):
    sharing_helper.share_resource(user, resource, receiver, permissions, receiver_type)


@Given('user "|any|" has created the following public link share in the server')
def step(context, user):
    data = {
        'resource': None,
        'permissions': None,
        'name': None,
        'password': None,
        'expireDate': None,
    }
    for row in context.table:
        key, value = row
        if key in data:
            data[key] = value
    sharing_helper.create_link_share(
        user,
        data['resource'],
        data['permissions'],
        data['name'],
        data['password'],
        data['expireDate'],
    )


@Then('user "|any|" should have a share with these details in the server:')
def step(context, user):
    path = None
    share_type = None
    share_with = None
    for key, value in context.table[1:]:
        if key == 'path':
            path = value
        elif key == 'share_type':
            share_type = value
        elif key == 'share_with':
            share_with = value

    share = sharing_helper.get_share(user, path, share_type, share_with)

    for key, value in context.table[1:]:
        if key == 'permissions':
            value = sharing_helper.get_permission_value(value)
        if key == 'share_type':
            value = sharing_helper.share_types[value]
        assert share.get(key) == value, 'Key value did not match'


@Given('user "|any|" has uploaded file "|any|" to "|any|" in the server')
def step(context, user, file_name, destination):
    if not is_owncloud_client():
        ext.upload_file(user, destination)
    else:
        webdav.upload_file(user, file_name, destination)



import os
import squish

from pageObjects.PublicLinkDialog import PublicLinkDialog
from pageObjects.SharingDialog import SharingDialog

from helpers.SetupClientHelper import get_resource_path, substitute_inline_codes
from helpers.FilesHelper import sanitize_path
from helpers.SyncHelper import (
    get_socket_connection,
    wait_for_resource_to_have_sync_status,
    SYNC_STATUS,
)
from helpers.ConfigHelper import get_config


def send_share_command(resource):
    socket_connect = get_socket_connection()
    socket_connect.sendCommand(f'SHARE:{resource}\n')
    if not socket_connect.read_socket_data_with_timeout(0.1):
        return False
    for line in socket_connect.get_available_responses():
        if line.startswith('SHARE:OK') and line.endswith(resource):
            return True
    return False


def open_sharing_dialog(resource):
    resource = get_resource_path(resource)
    resource_exist = squish.waitFor(
        lambda: os.path.exists(resource), get_config('minSyncTimeout') * 1000
    )
    if not resource_exist:
        raise FileNotFoundError(f"{resource} doesn't exists")
    share_dialog = squish.waitFor(
        lambda: send_share_command(resource),
        get_config('minSyncTimeout') * 1000,
    )
    if not share_dialog:
        raise LookupError(f"Sharing dialog didn't open for {resource}")


def create_public_link_share(resource, password='', permissions='', expire_date=''):
    open_sharing_dialog(resource)
    PublicLinkDialog.open_public_link_tab()
    PublicLinkDialog.create_public_link(password, permissions, expire_date)


def create_public_share_with_role(resource, role):
    resource = sanitize_path(substitute_inline_codes(resource))
    open_sharing_dialog(resource)
    PublicLinkDialog.open_public_link_tab()
    PublicLinkDialog.create_public_link_with_role(role)


def collaborator_should_be_listed(receiver, resource, permissions, receiver_count=0):
    # wait for client to the view
    wait_for_resource_to_have_sync_status(
        get_resource_path(), 'FOLDER', SYNC_STATUS['UPDATE']
    )
    open_sharing_dialog(resource)
    check_collaborator_and_premissions(receiver, permissions, receiver_count)
    SharingDialog.close_sharing_dialog()


def check_collaborator_and_premissions(receiver, permissions, receiver_count=0):
    permissions_list = permissions.split(',')

    shared_with_obj = SharingDialog.get_collaborators()

    #     we use sharedWithObj list from above while verifying if users are listed or not.
    #     For this we need an index value i.e receiver_count
    #     For 1st user in the list the index will be 0 which is receiver_count default value
    #     For 2nd user in the list the index will be 1 and so on

    test.compare(str(shared_with_obj[receiver_count].text), receiver)
    test.compare(
        SharingDialog.has_edit_permission(),
        ('edit' in permissions_list),
    )
    test.compare(
        SharingDialog.has_share_permission(),
        ('share' in permissions_list),
    )


@When(
    'the user adds "|any|" as collaborator of resource "|any|" with permissions "|any|" using the client-UI'
)
def step(context, receiver, resource, permissions):
    open_sharing_dialog(resource)
    SharingDialog.add_collaborator(receiver, permissions)
    SharingDialog.close_sharing_dialog()


@When('the user adds following collaborators of resource "|any|" using the client-UI')
def step(context, resource):
    open_sharing_dialog(resource)

    # In the following loop we are trying to share resource with given permission
    # to one user at a timegiven from the data table in the feature file
    for count, row in enumerate(context.table[1:]):
        receiver = row[0]
        permissions = row[1]
        SharingDialog.add_collaborator(receiver, permissions, False, count + 1)

    SharingDialog.close_sharing_dialog()


@When(
    'the user selects "|any|" as collaborator of resource "|any|" using the client-UI'
)
def step(context, receiver, resource):
    open_sharing_dialog(resource)
    SharingDialog.select_collaborator(receiver)


@When(
    'the user adds group "|any|" as collaborator of resource "|any|" with permissions "|any|" using the client-UI'
)
def step(context, receiver, resource, permissions):
    open_sharing_dialog(resource)
    SharingDialog.add_collaborator(receiver, permissions, True)
    SharingDialog.close_sharing_dialog()


@Then(
    'user "|any|" should be listed in the collaborators list for file "|any|" with permissions "|any|" on the client-UI'
)
def step(context, receiver, resource, permissions):
    collaborator_should_be_listed(receiver, resource, permissions)


@Then(
    # pylint: disable=line-too-long
    'group "|any|" should be listed in the collaborators list for file "|any|" with permissions "|any|" on the client-UI'
)
def step(context, receiver, resource, permissions):
    receiver += ' (group)'
    collaborator_should_be_listed(receiver, resource, permissions)


@When('the user opens the public links dialog of "|any|" using the client-UI')
def step(context, resource):
    open_sharing_dialog(resource)
    PublicLinkDialog.open_public_link_tab()


@When('the user toggles the password protection using the client-UI')
def step(context):
    PublicLinkDialog.toggle_password()


@Then('the password progress indicator should not be visible in the client-UI')
def step(context):
    squish.waitFor(lambda: (test.vp('publicLinkPasswordProgressIndicatorInvisible')))


@Then(
    'the password progress indicator should not be visible in the client-UI - expected to fail'
)
def step(context):
    squish.waitFor(lambda: (test.xvp('publicLinkPasswordProgressIndicatorInvisible')))


@When('the user opens the sharing dialog of "|any|" using the client-UI')
def step(context, resource):
    open_sharing_dialog(resource)


@Then('the text "|any|" should be displayed in the sharing dialog')
def step(context, message):
    test.compare(
        SharingDialog.get_sharing_dialog_message(),
        message,
    )


@Then('the error text "|any|" should be displayed in the sharing dialog')
def step(context, error_message):
    test.compare(
        SharingDialog.get_sharing_dialog_message(),
        error_message,
    )


@When(
    'the user creates a new public link for file "|any|" without password using the client-UI'
)
def step(context, resource):
    create_public_link_share(resource)


@When(
    'the user creates a new public link for file "|any|" with password "|any|" using the client-UI'
)
def step(context, resource, password):
    create_public_link_share(resource, password)


@Then('the expiration date of the last public link of file "|any|" should be "|any|"')
def step(context, resource, expiry_date):
    # wait for client to the view
    wait_for_resource_to_have_sync_status(
        get_resource_path(), 'FOLDER', SYNC_STATUS['UPDATE']
    )
    open_sharing_dialog(resource)
    PublicLinkDialog.open_public_link_tab()

    if expiry_date.strip('%') == 'default':
        expiry_date = PublicLinkDialog.get_default_expiry_date()
    actual_expiry_date = PublicLinkDialog.get_expiration_date()
    test.compare(expiry_date, actual_expiry_date)

    SharingDialog.close_sharing_dialog()


@When('the user edits the public link named "|any|" of file "|any|" changing following')
def step(context, _, __):
    expire_date = ''
    for row in context.table:
        if row[0] == 'expireDate':
            expire_date = row[1]
            break
    PublicLinkDialog.set_expiration_date(expire_date)


@When(
    # pylint: disable=line-too-long
    'the user creates a new public link with permissions "|any|" for folder "|any|" without password using the client-UI'
)
def step(context, permissions, resource):
    create_public_link_share(resource, '', permissions)


@When(
    # pylint: disable=line-too-long
    'the user creates a new public link with permissions "|any|" for folder "|any|" with password "|any|" using the client-UI'
)
def step(context, permissions, resource, password):
    create_public_link_share(resource, password, permissions)


@When('the user creates a new public link with following settings using the client-UI:')
def step(context):
    link_settings = {}
    for row in context.table:
        link_settings[row[0]] = row[1]

    if 'path' not in link_settings:
        raise ValueError("'path' is required but not given.")

    if 'expireDate' in link_settings and link_settings['expireDate'] == '%default%':
        link_settings['expireDate'] = link_settings['expireDate'].strip('%')

    create_public_link_share(
        resource=link_settings['path'],
        password=link_settings['password'] if 'password' in link_settings else None,
        expire_date=(
            link_settings['expireDate'] if 'expireDate' in link_settings else None
        ),
    )


@When(
    'the user creates a new public link for folder "|any|" using the client-UI with these details:'
)
def step(context, resource):
    role = ''
    for row in context.table:
        if row[0] == 'role':
            role = row[1]
            break

    if not role:
        raise ValueError('No role has been found')

    create_public_share_with_role(resource, role)


@When(
    'the user removes permissions "|any|" for user "|any|" of resource "|any|" using the client-UI'
)
def step(context, permissions, _, resource):
    open_sharing_dialog(resource)
    SharingDialog.remove_permissions(permissions)


@Step('the user closes the sharing dialog')
def step(context):
    SharingDialog.close_sharing_dialog()


@Then(
    '"|any|" permissions should not be displayed for user "|any|" for resource "|any|" on the client-UI'
)
def step(context, permissions, _, __):
    permissions_list = permissions.split(',')

    edit_checked, share_checked = SharingDialog.get_available_permission()

    if 'edit' in permissions_list:
        test.compare(edit_checked, False)

    if 'share' in permissions_list:
        test.compare(share_checked, False)


@Then('the error "|any|" should be displayed in the sharing dialog')
def step(context, error_message):
    test.compare(SharingDialog.get_error_text(), error_message)


@When(
    'the user tires to share resource "|any|" with the group "|any|" using the client-UI'
)
def step(context, resource, group):
    open_sharing_dialog(resource)

    SharingDialog.select_collaborator(group, True)


@When(
    'the user unshares the resource "|any|" for collaborator "|any|" using the client-UI'
)
def step(context, resource, _):
    open_sharing_dialog(resource)
    SharingDialog.unshare_with()


@When('the user deletes the public link for file "|any|"')
def step(context, resource):
    open_sharing_dialog(resource)
    PublicLinkDialog.open_public_link_tab()
    PublicLinkDialog.delete_public_link()


@When(
    'the user changes the password of public link "|any|" to "|any|" using the client-UI'
)
def step(context, _, password):
    PublicLinkDialog.change_password(password)


@Then(
    'the following users should be listed in as collaborators for file "|any|" on the client-UI'
)
def step(context, resource):
    # wait for client to the view
    wait_for_resource_to_have_sync_status(
        get_resource_path(), 'FOLDER', SYNC_STATUS['UPDATE']
    )
    open_sharing_dialog(resource)
    #     Here we are trying to verify if the user added in when step are listed in the client-UI or not
    #     We now have a variable name receiverCount which is used in collaborator_should_be_listed function call
    receiver_count = 0
    for row in context.table[1:]:
        receiver = row[0]
        permissions = row[1]

        check_collaborator_and_premissions(receiver, permissions, receiver_count)
        receiver_count += 1

    SharingDialog.close_sharing_dialog()


@When('the user searches for collaborator "|any|" using the client-UI')
def step(context, collaborator):
    SharingDialog.search_collaborator(collaborator)


@When(
    'the user searches for collaborator with autocomplete characters "|any|" using the client-UI'
)
def step(context, collaborator):
    SharingDialog.search_collaborator(collaborator)


@Then('the following users should be listed as suggested collaborators:')
def step(context):
    for collaborator in context.table[1:]:
        test.compare(
            SharingDialog.is_user_in_suggestion_list(collaborator[0]),
            True,
            "Assert user '" + collaborator[0] + "' is listed",
        )


@Then('the collaborators should be listed in the following order:')
def step(context):
    for index, collaborator in enumerate(context.table[1:], start=1):
        test.compare(
            SharingDialog.get_collaborator_name(index),
            collaborator[0],
        )



import names
import squish
from objectmaphelper import RegularExpression

from helpers.FilesHelper import build_conflicted_regex
from helpers.ConfigHelper import get_config


class Activity:
    TAB_CONTAINER = {
        "container": names.settings_dialogStack_QStackedWidget,
        "type": "QTabWidget",
        "visible": 1,
    }
    SUBTAB_CONTAINER = {
        "container": names.settings_dialogStack_QStackedWidget,
        "name": "qt_tabwidget_tabbar",
        "type": "QTabBar",
        "visible": 1,
    }
    NOT_SYNCED_TABLE = {
        "container": names.qt_tabwidget_stackedwidget_OCC_IssuesWidget_OCC_IssuesWidget,
        "name": "_tableView",
        "type": "QTableView",
        "visible": 1,
    }
    LOCAL_ACTIVITY_FILTER_BUTTON = {
        "container": names.qt_tabwidget_stackedwidget_OCC_ProtocolWidget_OCC_ProtocolWidget,
        "name": "_filterButton",
        "type": "QPushButton",
        "visible": 1,
    }
    SYNCED_ACTIVITY_FILTER_OPTION_SELECTOR = {
        "type": "QMenu",
        "unnamed": 1,
        "visible": 1,
        "window": names.settings_OCC_SettingsDialog,
    }
    SYNCED_ACTIVITY_TABLE = {
        "container": names.qt_tabwidget_stackedwidget_OCC_ProtocolWidget_OCC_ProtocolWidget,
        "name": "_tableView",
        "type": "QTableView",
        "visible": 1,
    }
    NOT_SYNCED_FILTER_BUTTON = {
        "container": names.qt_tabwidget_stackedwidget_OCC_IssuesWidget_OCC_IssuesWidget,
        "name": "_filterButton",
        "type": "QPushButton",
        "visible": 1,
    }
    NOT_SYNCED_FILTER_OPTION_SELECTOR = {
        "type": "QMenu",
        "unnamed": 1,
        "visible": 1,
        "window": names.settings_OCC_SettingsDialog,
    }
    SYNCED_ACTIVITY_TABLE_HEADER_SELECTOR = {
        "container": names.oCC_ProtocolWidget_tableView_QTableView,
        "name": "ActivityListHeaderV2",
        "orientation": 1,
        "type": "OCC::ExpandingHeaderView",
        "visible": 1,
    }
    NOT_SYNCED_ACTIVITY_TABLE_HEADER_SELECTOR = {
        "container": names.oCC_IssuesWidget_tableView_QTableView,
        "name": "ActivityErrorListHeaderV2",
        "orientation": 1,
        "type": "OCC::ExpandingHeaderView",
        "visible": 1,
    }

    @staticmethod
    def get_tab_object(tab_index):
        return {
            "container": Activity.SUBTAB_CONTAINER,
            "index": tab_index,
            "type": "TabItem",
        }

    @staticmethod
    def get_tab_text(tab_index):
        return squish.waitForObjectExists(Activity.get_tab_object(tab_index)).text

    @staticmethod
    def get_not_synced_file_selector(resource):
        return {
            "column": 1,
            "container": Activity.NOT_SYNCED_TABLE,
            "text": resource,
            "type": "QModelIndex",
        }

    @staticmethod
    def get_not_synced_status(row):
        return squish.waitForObjectExists(
            {
                "column": 6,
                "row": row,
                "container": Activity.NOT_SYNCED_TABLE,
                "type": "QModelIndex",
            }
        ).text

    @staticmethod
    def click_tab(tab_name):
        tab_found = False

        # NOTE: Some activity tabs are loaded dynamically
        # and the tab index changes after all the tabs are loaded properly
        # So wait for a second to let the UI render the tabs properly
        # before trying to click the tab
        squish.snooze(get_config("lowestSyncTimeout"))

        # Selecting tab by name fails for "Not Synced" when there are no unsynced files
        # Because files count will be appended like "Not Synced (2)"
        # So to overcome this the following approach has been implemented
        tab_count = squish.waitForObjectExists(Activity.SUBTAB_CONTAINER).count
        tabs = []
        for index in range(tab_count):
            tab_text = Activity.get_tab_text(index)
            tabs.append(tab_text)

            if tab_name in tab_text:
                tab_found = True
                # click_tab becomes flaky with "Not Synced" tab
                # because the tab text changes. e.g. "Not Synced (2)"
                # squish.click_tab(Activity.TAB_CONTAINER, tab_text)

                # NOTE: If only the objectOrName is specified,
                # the object is clicked in the middle by the Qt::LeftButton button
                # and with no keyboard modifiers pressed.
                squish.mouseClick(
                    squish.waitForObjectExists(Activity.get_tab_object(index))
                )
                break

        if not tab_found:
            raise LookupError(
                "Tab not found: "
                + tab_name
                + " in "
                + str(tabs)
                + ". Tabs count: "
                + str(tab_count)
            )

    @staticmethod
    def check_file_exist(filename):
        squish.waitForObjectExists(
            Activity.get_not_synced_file_selector(
                RegularExpression(build_conflicted_regex(filename))
            )
        )

    @staticmethod
    def is_resource_blacklisted(filename):
        result = squish.waitFor(
            lambda: Activity.has_sync_status(filename, "Blacklisted"),
            get_config("maxSyncTimeout") * 1000,
        )
        return result

    @staticmethod
    def is_resource_ignored(filename):
        result = squish.waitFor(
            lambda: Activity.has_sync_status(filename, "File Ignored"),
            get_config("maxSyncTimeout") * 1000,
        )
        return result

    @staticmethod
    def is_resource_excluded(filename):
        result = squish.waitFor(
            lambda: Activity.has_sync_status(filename, "Excluded"),
            get_config("maxSyncTimeout") * 1000,
        )
        return result

    @staticmethod
    def has_sync_status(filename, status):
        try:
            file_row = squish.waitForObject(
                Activity.get_not_synced_file_selector(filename),
                get_config("lowestSyncTimeout") * 1000,
            )["row"]
            if Activity.get_not_synced_status(file_row) == status:
                return True
            return False
        except:
            return False

    @staticmethod
    def select_synced_filter(sync_filter):
        squish.clickButton(squish.waitForObject(Activity.LOCAL_ACTIVITY_FILTER_BUTTON))
        squish.activateItem(
            squish.waitForObjectItem(
                Activity.SYNCED_ACTIVITY_FILTER_OPTION_SELECTOR, sync_filter
            )
        )

    @staticmethod
    def get_synced_file_selector(resource):
        return {
            "column": Activity.get_synced_table_column_number_by_name("File"),
            "container": Activity.SYNCED_ACTIVITY_TABLE,
            "text": resource,
            "type": "QModelIndex",
        }

    @staticmethod
    def get_synced_table_column_number_by_name(column_name):
        return squish.waitForObject(
            {
                "container": Activity.SYNCED_ACTIVITY_TABLE_HEADER_SELECTOR,
                "text": column_name,
                "type": "HeaderViewItem",
                "visible": True,
            }
        )["section"]

    @staticmethod
    def check_synced_table(resource, action, account):
        try:
            file_row = squish.waitForObject(
                Activity.get_synced_file_selector(resource),
                get_config("lowestSyncTimeout") * 1000,
            )["row"]
            squish.waitForObjectExists(
                {
                    "column": Activity.get_synced_table_column_number_by_name("Action"),
                    "row": file_row,
                    "container": Activity.SYNCED_ACTIVITY_TABLE,
                    "text": action,
                    "type": "QModelIndex",
                }
            )
            squish.waitForObjectExists(
                {
                    "column": Activity.get_synced_table_column_number_by_name(
                        "Account"
                    ),
                    "row": file_row,
                    "container": Activity.SYNCED_ACTIVITY_TABLE,
                    "text": account,
                    "type": "QModelIndex",
                }
            )
            return True
        except:
            return False

    @staticmethod
    def select_not_synced_filter(filter_option):
        squish.clickButton(squish.waitForObject(Activity.NOT_SYNCED_FILTER_BUTTON))
        squish.activateItem(
            squish.waitForObjectItem(
                Activity.NOT_SYNCED_FILTER_OPTION_SELECTOR, filter_option
            )
        )

    @staticmethod
    def get_not_synced_table_column_number_by_name(column_name):
        return squish.waitForObject(
            {
                "container": Activity.NOT_SYNCED_ACTIVITY_TABLE_HEADER_SELECTOR,
                "text": column_name,
                "type": "HeaderViewItem",
                "visible": True,
            }
        )["section"]

    @staticmethod
    def check_not_synced_table(resource, status, account):
        try:
            file_row = squish.waitForObject(
                Activity.get_not_synced_file_selector(resource),
                get_config("lowestSyncTimeout") * 1000,
            )["row"]
            squish.waitForObjectExists(
                {
                    "column": Activity.get_not_synced_table_column_number_by_name(
                        "Status"
                    ),
                    "row": file_row,
                    "container": Activity.NOT_SYNCED_TABLE,
                    "text": status,
                    "type": "QModelIndex",
                }
            )
            squish.waitForObjectExists(
                {
                    "column": Activity.get_not_synced_table_column_number_by_name(
                        "Account"
                    ),
                    "row": file_row,
                    "container": Activity.NOT_SYNCED_TABLE,
                    "text": account,
                    "type": "QModelIndex",
                }
            )
            return True
        except:
            return False



import names
import squish
import object  # pylint: disable=redefined-builtin

from helpers.SetupClientHelper import wait_until_app_killed
from helpers.ConfigHelper import get_config


class Toolbar:
    TOOLBAR_ROW = {
        "container": names.dialogStack_quickWidget_OCC_QmlUtils_OCQuickWidget,
        "type": "RowLayout",
        "visible": True,
    }
    ACCOUNT_BUTTON = {
        "checkable": False,
        "container": names.dialogStack_quickWidget_OCC_QmlUtils_OCQuickWidget,
        "type": "AccountButton",
        "visible": True,
    }
    ADD_ACCOUNT_BUTTON = {
        "container": names.dialogStack_quickWidget_QQuickWidget,
        "id": "addAccountButton",
        "type": "AccountButton",
        "visible": True,
    }
    ACTIVITY_BUTTON = {
        "container": names.dialogStack_quickWidget_QQuickWidget,
        "id": "logButton",
        "type": "AccountButton",
        "visible": True,
    }
    SETTINGS_BUTTON = {
        "container": names.dialogStack_quickWidget_QQuickWidget,
        "id": "settingsButton",
        "type": "AccountButton",
        "visible": True,
    }
    QUIT_BUTTON = {
        "container": names.dialogStack_quickWidget_QQuickWidget,
        "id": "quitButton",
        "type": "AccountButton",
        "visible": True,
    }
    QUIT_CONFIRMATION_DIALOG = {
        "type": "QMessageBox",
        "unnamed": 1,
        "visible": 1,
    }
    CONFIRM_QUIT_BUTTON = {
        "text": "Yes",
        "type": "QPushButton",
        "unnamed": 1,
        "visible": 1,
        "window": QUIT_CONFIRMATION_DIALOG,
    }

    TOOLBAR_ITEMS = ["Add Account", "Activity", "Settings", "Quit"]

    @staticmethod
    def get_item_selector(item_name):
        return {
            "container": names.dialogStack_quickWidget_QQuickWidget,
            "text": item_name,
            "type": "Label",
            "visible": True,
        }

    @staticmethod
    def has_item(item_name, timeout=get_config("minSyncTimeout") * 1000):
        try:
            squish.waitForObject(Toolbar.get_item_selector(item_name), timeout)
            return True
        except:
            return False

    @staticmethod
    def open_activity():
        squish.mouseClick(squish.waitForObject(Toolbar.ACTIVITY_BUTTON))

    @staticmethod
    def open_new_account_setup():
        squish.mouseClick(squish.waitForObject(Toolbar.ADD_ACCOUNT_BUTTON))

    @staticmethod
    def open_account(displayname):
        _, selector = Toolbar.get_account(displayname)
        squish.mouseClick(squish.waitForObject(selector))

    @staticmethod
    def get_displayed_account_text(displayname, host):
        return str(
            squish.waitForObjectExists(
                Toolbar.get_item_selector(displayname + "\n" + host)
            ).text
        )

    @staticmethod
    def open_settings_tab():
        squish.mouseClick(squish.waitForObject(Toolbar.SETTINGS_BUTTON))

    @staticmethod
    def quit_owncloud():
        squish.mouseClick(squish.waitForObject(Toolbar.QUIT_BUTTON))
        squish.clickButton(squish.waitForObject(Toolbar.CONFIRM_QUIT_BUTTON))
        for ctx in squish.applicationContextList():
            pid = ctx.pid
            ctx.detach()
            wait_until_app_killed(pid)

    @staticmethod
    def get_accounts():
        accounts = {}
        selectors = {}
        children_obj = object.children(squish.waitForObjectExists(Toolbar.TOOLBAR_ROW))
        account_idx = 1
        for obj in children_obj:
            if hasattr(obj, "accountState"):
                account_info = {
                    "displayname": str(obj.accountState.account.davDisplayName),
                    "username": str(obj.accountState.account.davUser),
                    "hostname": str(obj.accountState.account.hostName),
                    "initials": str(obj.accountState.account.initials),
                    "current": obj.checked,
                }
                account_locator = Toolbar.ACCOUNT_BUTTON.copy()
                if account_idx > 1:
                    account_locator.update({"occurrence": account_idx})
                account_locator.update({"text": account_info["hostname"]})

                accounts[account_info["displayname"]] = account_info
                selectors[account_info["displayname"]] = account_locator
                account_idx += 1
        return accounts, selectors

    @staticmethod
    def get_account(display_name):
        accounts, selectors = Toolbar.get_accounts()
        return accounts.get(display_name), selectors.get(display_name)

    @staticmethod
    def get_active_account():
        accounts, selectors = Toolbar.get_accounts()
        for account, info in accounts.items():
            if info["current"]:
                return info, selectors[account]
        return None, None

    @staticmethod
    def account_has_focus(display_name):
        account, selector = Toolbar.get_account(display_name)
        return account["current"] and squish.waitForObjectExists(selector).checked

    @staticmethod
    def account_exists(display_name):
        account, selector = Toolbar.get_account(display_name)
        if (
            account is None
            or selector is None
            and account["displayname"] != display_name
        ):
            raise LookupError(f'Account "{display_name}" does not exist')
        squish.waitForObjectExists(selector)



import json

import helpers.api.http_helper as request
from helpers.api.utils import url_join
from helpers.ConfigHelper import get_config


def get_graph_url():
    return url_join(get_config("localBackendUrl"), "graph", "v1.0")


def create_group(group_name):
    url = url_join(get_graph_url(), "groups")
    body = json.dumps({"displayName": group_name})
    response = request.post(url, body)
    request.assert_http_status(response, 201, f"Failed to create group '{group_name}'")
    resp_object = response.json()
    return {
        "id": resp_object["id"],
        "displayName": resp_object["displayName"],
    }


def delete_group(group_id):
    url = url_join(get_graph_url(), "groups", group_id)
    response = request.delete(url)
    request.assert_http_status(response, 200, f"Failed to delete group '{group_id}'")


def get_group_id(group_name):
    url = url_join(get_graph_url(), "groups", group_name)
    response = request.get(url)
    request.assert_http_status(response, 200, f"Failed to get group '{group_name}'")
    resp_object = response.json()
    return resp_object["id"]


def get_user_id(user):
    url = url_join(get_graph_url(), "users", user)
    response = request.get(url)
    request.assert_http_status(response, 200, f"Failed to get user '{user}'")
    resp_object = response.json()
    return resp_object["id"]


def add_user_to_group(user, group_name):
    url = url_join(
        get_graph_url(), "groups", get_group_id(group_name), "members", "$ref"
    )
    data = url_join(get_graph_url(), "users", get_user_id(user))
    body = json.dumps({"@odata.id": data})
    response = request.post(url, body)
    request.assert_http_status(
        response, 204, f"Failed to add user '{user}' to group '{group_name}'"
    )


def create_user(username, password, displayname, email):
    url = url_join(get_graph_url(), "users")
    body = json.dumps(
        {
            "onPremisesSamAccountName": username,
            "passwordProfile": {"password": password},
            "displayName": displayname,
            "mail": email,
        }
    )
    response = request.post(url, body)
    request.assert_http_status(response, 201, f"Failed to create user '{username}'")
    resp_object = response.json()
    return {
        "id": resp_object["id"],
        "username": username,
        "password": password,
        "displayname": resp_object["displayName"],
        "email": resp_object["mail"],
    }


def delete_user(user_id):
    url = url_join(get_graph_url(), "users", user_id)
    response = request.delete(url)
    request.assert_http_status(response, 204, "Failed to delete user")



import os
import platform
import builtins
from tempfile import gettempdir
from configparser import ConfigParser
from pathlib import Path

CURRENT_DIR = Path(__file__).resolve().parent


def read_env_file():
    envs = {}
    script_path = os.path.dirname(os.path.realpath(__file__))
    env_path = os.path.abspath(os.path.join(script_path, '..', '..', '..', 'envs.txt'))
    with open(env_path, 'rt', encoding='UTF-8') as f:
        for line in f:
            if not line.strip():
                continue
            if line.startswith('#'):
                continue
            key, value = line.split('=', 1)
            envs[key] = value.strip()
    return envs


def get_config_from_env_file(env):
    envs = read_env_file()
    if env in envs:
        return envs[env]
    raise KeyError(f'Environment "{env}" not found in envs.txt')


def is_windows():
    return platform.system() == 'Windows'


def is_linux():
    return platform.system() == 'Linux'


def get_win_user_home():
    return os.environ.get('UserProfile')


def get_client_root_path():
    if is_windows():
        return os.path.join(get_win_user_home(), 'owncloudtest')
    return os.path.join(gettempdir(), 'owncloudtest')


def get_test_config_file_path():
    script_path = os.path.dirname(os.path.realpath(__file__))
    return os.path.abspath(os.path.join(script_path, '..', '..', '..', 'config.ini'))


def get_client_name():
    if os.environ.get('CLIENT_NAME'):
        return os.environ.get('CLIENT_NAME')
    cfg = ConfigParser()
    if cfg.read(get_test_config_file_path()) and cfg.get('DEFAULT', 'CLIENT_NAME'):
        return cfg.get('DEFAULT', 'CLIENT_NAME')
    return 'ownCloud'


def get_config_home():
    if is_windows():
        # There is no way to set custom config path in windows
        # TODO: set to different path if option is available
        return os.path.join(
            get_win_user_home(), 'AppData', 'Roaming', get_client_name()
        )
    return os.path.join(get_config_from_env_file('XDG_CONFIG_HOME'), get_client_name())


def get_default_home_dir():
    if is_windows():
        return get_win_user_home()
    return os.environ.get('HOME')


# map environment variables to config keys
CONFIG_ENV_MAP = {
    'localBackendUrl': 'BACKEND_HOST',
    'secureLocalBackendUrl': 'SECURE_BACKEND_HOST',
    'maxSyncTimeout': 'MAX_SYNC_TIMEOUT',
    'minSyncTimeout': 'MIN_SYNC_TIMEOUT',
    'lowestSyncTimeout': 'LOWEST_SYNC_TIMEOUT',
    'clientLogFile': 'CLIENT_LOG_FILE',
    'clientRootSyncPath': 'CLIENT_ROOT_SYNC_PATH',
    'tempFolderPath': 'TEMP_FOLDER_PATH',
    'clientConfigDir': 'CLIENT_CONFIG_DIR',
    'guiTestReportDir': 'GUI_TEST_REPORT_DIR',
    'predefined_users': 'PREDEFINED_USERS',
    'personal_sync_folder': 'PERSONAL_SYNC_FOLDER',
    'ocis': 'OCIS',
    'record_video_on_failure': 'RECORD_VIDEO_ON_FAILURE',
}

DEFAULT_PATH_CONFIG = {
    'custom_lib': os.path.abspath('../shared/scripts/custom_lib'),
    'home_dir': get_default_home_dir(),
    # allow to record first 5 videos
    'video_record_limit': 5,
}

# default config values
CONFIG = {
    'localBackendUrl': 'https://localhost:9200/',
    'secureLocalBackendUrl': 'https://localhost:9200/',
    'maxSyncTimeout': 60,
    'minSyncTimeout': 5,
    'lowestSyncTimeout': 1,
    'clientLogFile': '-',
    'clientRootSyncPath': get_client_root_path(),
    'tempFolderPath': os.path.join(get_client_root_path(), 'temp'),
    'clientConfigDir': get_config_home(),
    'guiTestReportDir': os.path.abspath('../reports'),
    'predefined_users': '',
    'client_name': get_client_name(),
    'personal_sync_folder': 'Personal',
    'ocis': False,
    'record_video_on_failure': False,
    'files_for_upload': os.path.join(CURRENT_DIR.parent.parent, 'files-for-upload'),
}
CONFIG.update(DEFAULT_PATH_CONFIG)

READONLY_CONFIG = list(CONFIG_ENV_MAP.keys()) + list(DEFAULT_PATH_CONFIG.keys())

SCENARIO_CONFIGS = {}


def read_cfg_file(cfg_path):
    cfg = ConfigParser()
    if cfg.read(cfg_path):
        for key, _ in CONFIG.items():
            if key in CONFIG_ENV_MAP:
                if value := cfg.get('DEFAULT', CONFIG_ENV_MAP[key]):
                    if key in ('ocis', 'record_video_on_failure'):
                        CONFIG[key] = value == 'true'
                    else:
                        CONFIG[key] = value


def init_config():
    # try reading configs from config.ini
    try:
        read_cfg_file(get_test_config_file_path())
    except:
        pass

    # read and override configs from environment variables
    for key, value in CONFIG_ENV_MAP.items():
        if os.environ.get(value):
            if key in ('ocis', 'record_video_on_failure'):
                CONFIG[key] = os.environ.get(value) == 'true'
            else:
                CONFIG[key] = os.environ.get(value)

    # Set the default values if empty
    for key, value in CONFIG.items():
        if key in ('maxSyncTimeout', 'minSyncTimeout'):
            CONFIG[key] = builtins.int(value)
        elif key in ('localBackendUrl', 'secureLocalBackendUrl'):
            # make sure there is always one trailing slash
            CONFIG[key] = value.rstrip('/') + '/'
        elif key in (
            'clientRootSyncPath',
            'tempFolderPath',
            'clientConfigDir',
            'guiTestReportDir',
        ):
            # make sure there is always one trailing slash
            if is_windows():
                value = value.replace('/', '\\')
                CONFIG[key] = value.rstrip('\\') + '\\'
            else:
                CONFIG[key] = value.rstrip('/') + '/'


def get_config(key):
    return CONFIG[key]


def set_config(key, value):
    if key in READONLY_CONFIG:
        raise KeyError(f'Cannot set read-only config: {key}')
    # save the initial config value
    if key not in SCENARIO_CONFIGS:
        SCENARIO_CONFIGS[key] = CONFIG.get(key)
    CONFIG[key] = value


def clear_scenario_config():
    for key, value in SCENARIO_CONFIGS.items():
        CONFIG[key] = value


def is_owncloud_client():
    return get_config('client_name') == 'ownCloud'



import names
import squish


from helpers.WebUIHelper import authorize_via_webui
from helpers.ConfigHelper import get_config


class EnterPassword:
    LOGIN_CONTAINER = {
        "name": "LoginRequiredDialog",
        "type": "OCC::LoginRequiredDialog",
        "visible": 1,
    }
    LOGIN_USER_LABEL = {
        "container": names.groupBox_OCC_QmlUtils_OCQuickWidget,
        "type": "Label",
        "visible": True,
    }
    USERNAME_BOX = {
        "name": "usernameLineEdit",
        "type": "QLineEdit",
        "visible": 1,
        "window": LOGIN_CONTAINER,
    }
    PASSWORD_BOX = {
        "container": names.groupBox_OCC_QmlUtils_OCQuickWidget,
        "id": "passwordField",
        "type": "TextField",
        "visible": True,
    }
    LOGIN_BUTTON = {
        "container": names.groupBox_OCC_QmlUtils_OCQuickWidget,
        "id": "loginButton",
        "type": "Button",
        "visible": True,
    }
    LOGOUT_BUTTON = {
        "container": names.groupBox_OCC_QmlUtils_OCQuickWidget,
        "id": "logOutButton",
        "type": "Button",
        "visible": True,
    }
    COPY_URL_TO_CLIPBOARD_BUTTON = {
        "container": names.groupBox_OCC_QmlUtils_OCQuickWidget,
        "id": "copyToClipboardButton",
        "type": "Button",
        "visible": True,
    }
    TLS_CERT_WINDOW = {
        "name": "OCC__TlsErrorDialog",
        "type": "OCC::TlsErrorDialog",
        "visible": 1,
    }
    ACCEPT_CERTIFICATE_YES = {
        "text": "Yes",
        "type": "QPushButton",
        "visible": 1,
        "window": TLS_CERT_WINDOW,
    }

    def __init__(self, occurrence=1):
        if occurrence > 1 and get_config("ocis"):
            self.TLS_CERT_WINDOW.update({"occurrence": occurrence})

    def get_username(self):
        # Parse username from the login label:
        label = str(squish.waitForObjectExists(self.LOGIN_USER_LABEL).text)
        username = label.split(" ", maxsplit=2)[1]
        return username.capitalize()

    def enter_password(self, password):
        squish.waitForObjectExists(
            self.PASSWORD_BOX, get_config("minSyncTimeout") * 1000
        )
        squish.mouseClick(squish.waitForObjectExists(self.PASSWORD_BOX))
        squish.nativeType(password)
        squish.mouseClick(squish.waitForObjectExists(self.LOGIN_BUTTON))

    def oidc_relogin(self, username, password):
        # wait 500ms for copy button to fully load
        squish.snooze(1 / 2)
        squish.mouseClick(squish.waitForObject(self.COPY_URL_TO_CLIPBOARD_BUTTON))
        authorize_via_webui(username, password)

    def oauth_relogin(self, username, password):
        # wait 500ms for copy button to fully load
        squish.snooze(1 / 2)
        squish.mouseClick(squish.waitForObject(self.COPY_URL_TO_CLIPBOARD_BUTTON))
        authorize_via_webui(username, password, "oauth")

    def relogin(self, username, password, oauth=False):
        if get_config("ocis"):
            self.oidc_relogin(username, password)
        elif oauth:
            self.oauth_relogin(username, password)
        else:
            self.enter_password(password)

    def login_after_setup(self, username, password):
        if get_config("ocis"):
            self.oidc_relogin(username, password)
        else:
            self.enter_password(password)

    def logout(self):
        squish.mouseClick(squish.waitForObject(self.LOGOUT_BUTTON))

    def accept_certificate(self):
        squish.clickButton(squish.waitForObject(self.ACCEPT_CERTIFICATE_YES))



import re
import squish

from pageObjects.SyncConnectionWizard import SyncConnectionWizard
from pageObjects.SyncConnection import SyncConnection
from pageObjects.Toolbar import Toolbar
from pageObjects.Activity import Activity
from pageObjects.Settings import Settings

from helpers.ConfigHelper import get_config, is_windows, set_config, is_owncloud_client
from helpers.SyncHelper import (
    wait_for_resource_to_sync,
    wait_for_resource_to_have_sync_error,
    wait_for_initial_sync_to_complete,
)
from helpers.SetupClientHelper import (
    get_temp_resource_path,
    set_current_user_sync_path,
    substitute_inline_codes,
    get_resource_path,
)
from helpers.UserHelper import get_displayname_for_user, get_username_for_user


@When('using sync connection folder "|any|"')
def step(context, sync_folder):
    if not is_owncloud_client():
        set_config(
            'syncConnectionName',
            f'{sync_folder} (Shared by {get_username_for_user("admin")})',
        )
    else:
        set_config('syncConnectionName', sync_folder)
        # wait for files to sync
        wait_for_initial_sync_to_complete(get_resource_path('/', space=sync_folder))


@Given('the user has paused the file sync')
def step(context):
    SyncConnection.pause_sync()


@When('the user resumes the file sync on the client')
def step(context):
    SyncConnection.resume_sync()


@When('the user force syncs the files')
def step(context):
    SyncConnection.force_sync()


@When('the user waits for the files to sync')
def step(context):
    wait_for_resource_to_sync(get_resource_path('/'))


@When(r'the user waits for (file|folder) "([^"]*)" to be synced', regexp=True)
def step(context, resource_type, resource):
    resource = get_resource_path(resource)
    wait_for_resource_to_sync(resource, resource_type)


@When(r'the user waits for (file|folder) "([^"]*)" to have sync error', regexp=True)
def step(context, resource_type, resource):
    resource = get_resource_path(resource)
    wait_for_resource_to_have_sync_error(resource, resource_type)


@When(
    r'user "([^"]*)" waits for (file|folder) "([^"]*)" to have sync error', regexp=True
)
def step(context, username, resource_type, resource):
    resource = get_resource_path(resource, username)
    wait_for_resource_to_have_sync_error(resource, resource_type)


@When('the user enables virtual file support')
def step(context):
    SyncConnection.enable_vfs()
    # TODO: remove snooze with proper wait
    # let the client re-sync
    squish.snooze(get_config('minSyncTimeout'))


@Then('the "|any|" button should be available')
def step(context, item):
    SyncConnection.open_menu()
    SyncConnection.has_menu_item(item)


@Then('the "|any|" button should not be available')
def step(context, item):
    SyncConnection.open_menu()
    test.compare(
        SyncConnection.menu_item_exists(item),
        False,
        f'Menu item "{item}" does not exist.',
    )


@When('the user disables virtual file support')
def step(context):
    SyncConnection.disable_vfs()
    # TODO: remove snooze with proper wait
    # let the client re-sync
    squish.snooze(get_config('minSyncTimeout'))


@When('the user clicks on the activity tab')
def step(context):
    Toolbar.open_activity()


@Then('the table of conflict warnings should include file "|any|"')
def step(context, filename):
    Activity.check_file_exist(filename)


@Then('the file "|any|" should be blacklisted')
def step(context, filename):
    test.compare(
        True, Activity.is_resource_blacklisted(filename), 'File is Blacklisted'
    )


@Then('the file "|any|" should be ignored')
def step(context, filename):
    test.compare(True, Activity.is_resource_ignored(filename), 'File is Ignored')


@Then('the file "|any|" should be excluded')
def step(context, filename):
    test.compare(True, Activity.is_resource_excluded(filename), 'File is Excluded')


@When('the user selects "|any|" tab in the activity')
def step(context, tab_name):
    Activity.click_tab(tab_name)


@Then('the toolbar should have the following tabs:')
def step(context):
    for tab_name in context.table:
        Toolbar.has_item(tab_name[0])


@When('the user selects the following folders to sync:')
def step(context):
    folders = []
    for row in context.table[1:]:
        folders.append(row[0])
    SyncConnectionWizard.select_folders_to_sync(folders)
    SyncConnectionWizard.add_sync_connection()


@When('the user sorts the folder list by "|any|"')
def step(context, header_text):
    if (header_text := header_text.capitalize()) in ['Size', 'Name']:
        SyncConnectionWizard.sort_by(header_text)
    else:
        raise ValueError("Sorting by '" + header_text + "' is not supported.")


@Then('the sync all checkbox should be checked')
def step(context):
    test.compare(
        SyncConnectionWizard.is_root_folder_checked(),
        True,
        'Sync all checkbox is checked',
    )


@Then('the folders should be in the following order:')
def step(context):
    row_index = 0
    for row in context.table[1:]:
        expected_folder = row[0]
        actual_folder = SyncConnectionWizard.get_item_name_from_row(row_index)
        test.compare(actual_folder, expected_folder)

        row_index += 1


@When('the user selects "|any|" space in sync connection wizard')
def step(context, space_name):
    if get_config('ocis'):
        if not is_owncloud_client():
            space_name = get_config('personal_sync_folder')
        SyncConnectionWizard.select_space(space_name)
        SyncConnectionWizard.next_step()
        set_config('syncConnectionName', space_name)


@When('the user sets the sync path in sync connection wizard')
def step(context):
    SyncConnectionWizard.set_sync_path()


@When(
    'the user sets the temp folder "|any|" as local sync path in sync connection wizard'
)
def step(context, folder_name):
    sync_path = get_temp_resource_path(folder_name)
    SyncConnectionWizard.set_sync_path(sync_path)
    if get_config('ocis'):
        # empty connection name when using temporary locations
        set_config('syncConnectionName', '')
        set_current_user_sync_path(sync_path)


@When('the user selects "|any|" as a remote destination folder')
def step(context, folder_name):
    # There's no remote destination section with oCIS server
    if not get_config('ocis'):
        if not is_owncloud_client():
            folder_name = get_config('client_name')
        SyncConnectionWizard.select_remote_destination_folder(folder_name)


@When('the user syncs the "|any|" space')
def step(context, space_name):
    if space_name == 'Personal' and not is_owncloud_client():
        space_name = get_config('personal_sync_folder')
    SyncConnectionWizard.sync_space(space_name)


@Then('the settings tab should have the following options in the general section:')
def step(context):
    for item in context.table:
        Settings.check_general_option(item[0])


@Then('the settings tab should have the following options in the advanced section:')
def step(context):
    for item in context.table:
        Settings.check_advanced_option(item[0])


@Then('the settings tab should have the following options in the network section:')
def step(context):
    for item in context.table:
        Settings.check_network_option(item[0])


@When('the user opens the about dialog')
def step(context):
    Settings.open_about_button()


@Then('the about dialog should be opened')
def step(context):
    Settings.wait_for_about_dialog_to_be_visible()


@When('the user adds the folder sync connection')
def step(context):
    SyncConnectionWizard.add_sync_connection()


@When('user unselects all the remote folders')
def step(context):
    SyncConnectionWizard.deselect_all_remote_folders()


@When('the user |word| VFS support for Windows')
def step(context, action):
    if is_windows():
        action = action.rstrip('s')
        SyncConnectionWizard.enable_disable_vfs_support(action)


@When('user unselects a folder "|any|" in selective sync')
def step(context, folder_name):
    SyncConnection.choose_what_to_sync()
    SyncConnection.unselect_folder_in_selective_sync(folder_name)


@Then('the sync folder list should be empty')
def step(context):
    test.compare(
        0,
        SyncConnection.get_folder_connection_count(),
        'Sync connections should be empty',
    )


@When('the user navigates back in the sync connection wizard')
def step(context):
    SyncConnectionWizard.back()


@When('the user creates a folder "|any|" in the remote destination wizard')
def step(context, folder_name):
    if not get_config('ocis'):
        SyncConnectionWizard.create_folder_in_remote_destination(folder_name)


@When('the user refreshes the remote destination in the sync connection wizard')
def step(context):
    if not get_config('ocis'):
        SyncConnectionWizard.refresh_remote()


@Then('the folder "|any|" should be present in the remote destination wizard')
def step(context, folder_name):
    if not get_config('ocis'):
        has_folder, folder_selector = SyncConnectionWizard.has_remote_folder(
            folder_name
        )
        test.compare(True, has_folder, 'Folder should be in the remote list')


@When('the user selects remove folder sync connection option')
def step(context):
    SyncConnection.remove_folder_sync_connection()


@When('the user cancels the folder sync connection removal dialog')
def step(context):
    SyncConnection.cancel_folder_sync_connection_removal()


@When('the user removes the "|any|" folder sync connection')
def step(context, sync_folder):
    SyncConnection.remove_folder_sync_connection(sync_folder)
    SyncConnection.confirm_folder_sync_connection_removal()


@Then('the file "|any|" should have status "|any|" in the activity tab')
def step(context, file_name, status):
    Activity.has_sync_status(file_name, status)


@When('the user opens the sync connection wizard')
def step(context):
    SyncConnectionWizard.open_sync_connection_wizard()


@Then('the button to open sync connection wizard should be disabled')
def step(context):
    test.compare(
        False,
        SyncConnectionWizard.is_add_sync_folder_button_enabled(),
        'Button to open sync connection wizard should be disabled',
    )


@When('the user checks the activities of account "|any|"')
def step(context, account):
    account = substitute_inline_codes(account)
    Activity.select_synced_filter(account)


@Then('the following activities should be displayed in synced table')
def step(context):
    for row in context.table[1:]:
        resource = row[0]
        action = row[1]
        account = substitute_inline_codes(row[2])
        test.compare(
            Activity.check_synced_table(resource, action, account),
            True,
            'Resource should be displayed in the synced table',
        )


@Then(
    r'the following activities (should|should not) be displayed in not synced table',
    regexp=True,
)
def step(context, should_or_should_not):
    expected = should_or_should_not == 'should'
    for row in context.table[1:]:
        resource = row[0]
        status = row[1]
        account = substitute_inline_codes(row[2])
        if not is_owncloud_client():
            # get the displayname of user from account string
            # and replace the existing displayname with predefined user's displayname from the account string
            displayname = get_displayname_for_user(account.split()[0].strip())
            account = re.sub(r'^[^@]+', displayname, account)
        test.compare(
            Activity.check_not_synced_table(resource, status, account),
            expected,
            'Resource should be displayed in the not synced table',
        )


@When('the user unchecks the "|any|" filter')
def step(context, filter_option):
    Activity.select_not_synced_filter(filter_option)


@Then('the space "|any|" should not exist in the sync folder list')
def step(context, space_name):
    test.compare(
        SyncConnectionWizard.is_space_available(space_name),
        False,
        f'Expected {space_name} to be not synced.',
    )



# -*- coding: utf-8 -*-

# This file contains hook functions to run as the .feature file is executed.
#
# A common use-case is to use the OnScenarioStart/OnScenarioEnd hooks to
# start and stop an AUT, e.g.
#
# @OnScenarioStart
# def hook(context):
#     startApplication("addressbook")
#
# @OnScenarioEnd
# def hook(context):
#     currentApplicationContext().detach()
#
# See the section 'Performing Actions During Test Execution Via Hooks' in the Squish
# manual for a complete reference of the available API.
import shutil
import os
from datetime import datetime

from helpers.StacktraceHelper import get_core_dumps, generate_stacktrace
from helpers.SyncHelper import close_socket_connection, clear_waited_after_sync
from helpers.SpaceHelper import delete_project_spaces
from helpers.api.provisioning import (
    delete_created_groups,
    delete_created_users,
    create_user,
)
from helpers.SetupClientHelper import wait_until_app_killed
from helpers.ConfigHelper import (
    init_config,
    get_config,
    set_config,
    clear_scenario_config,
    is_windows,
    is_linux,
    is_owncloud_client,
)
from helpers.FilesHelper import prefix_path_namespace, cleanup_created_paths
from helpers.ReportHelper import save_video_recording, take_screenshot, is_video_enabled
import helpers.api.oc10 as oc
from helpers.UserHelper import init_predefined_users
from helpers.api.external_api import (
    delete_all_resources,
    permanently_delete_all_resources,
    delete_all_spaces,
    permanently_delete_all_spaces,
)

from pageObjects.Toolbar import Toolbar
from pageObjects.AccountSetting import AccountSetting
from pageObjects.AccountConnectionWizard import AccountConnectionWizard

# Squish test settings:
# This controls whether a test (scenario) should stop execution on failure or not
# If set to True, the scenario will stop on the first step failure and remaining steps will not be executed
# If set to False, the scenario will continue to execute all steps and report all failures at the end
testSettings.throwOnFailure = True

# this will reset in every test suite
PREVIOUS_FAIL_RESULT_COUNT = 0
PREVIOUS_ERROR_RESULT_COUNT = 0


# runs before a feature
# Order: 1
@OnFeatureStart
def hook(context):
    init_config()
    if not is_owncloud_client():
        init_predefined_users()


# runs before every scenario
# Order: 1
@OnScenarioStart
def hook(context):
    clear_scenario_config()
    # create admin user
    if not is_owncloud_client():
        create_user("admin")


# runs before every scenario
# Order: 2
@OnScenarioStart
def hook(context):
    # set owncloud config file path
    config_dir = get_config("clientConfigDir")
    if os.path.exists(config_dir):
        if len(os.listdir(config_dir)) and is_windows():
            raise FileExistsError(
                "Looks like you have previous client config in '"
                + config_dir
                + "'\n[DANGER] Delete it and try again."
                + "\n[DANGER] Removing config file will make client to lost the previously added accounts."
            )
        # clean previous configs
        shutil.rmtree(config_dir)
    os.makedirs(config_dir, 0o0755)
    set_config(
        "clientConfigFile",
        os.path.join(config_dir, get_config("client_name").lower() + ".cfg"),
    )

    # create reports dir if not exists
    test_report_dir = get_config("guiTestReportDir")
    if not os.path.exists(test_report_dir):
        os.makedirs(test_report_dir)

    # log tests scenario title on serverlog file
    if os.getenv("CI"):
        with open(test_report_dir + "/serverlog.log", "a", encoding="utf-8") as f:
            f.write(
                str((datetime.now()).strftime("%H:%M:%S:%f"))
                + "\tBDD Scenario: "
                + context.title
                + "\n"
            )

    # this path will be changed according to the user added to the client
    # e.g.: /tmp/client-bdd/Alice
    set_config("currentUserSyncPath", "")

    root_sync_dir = get_config("clientRootSyncPath")
    if not os.path.exists(root_sync_dir):
        os.makedirs(root_sync_dir)

    tmp_dir = get_config("tempFolderPath")
    if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)

    # sync connection folder display name
    set_config(
        "syncConnectionName",
        get_config("personal_sync_folder"),
    )


# determines if the test scenario failed or not
# Currently, this workaround is needed because we cannot find out
#   a way to determine the pass/fail status of currently running test scenario.
# And, resultCount("errors")  and resultCount("fails")
#   return the total number of error/failed test scenarios of a test suite.
def scenario_failed():
    return (
        test.resultCount("fails") - PREVIOUS_FAIL_RESULT_COUNT > 0
        or test.resultCount("errors") - PREVIOUS_ERROR_RESULT_COUNT > 0
    )


def scenario_title_to_filename(title):
    # scenario name can have "/" which is invalid filename
    return title.replace(" ", "_").replace("/", "_").strip(".")


# runs after every scenario
# Order: 1
@OnScenarioEnd
def hook(context):
    clear_waited_after_sync()
    close_socket_connection()

    # generate screenshot and video reports
    if is_linux():
        filename = scenario_title_to_filename(context.title)
        if scenario_failed():
            take_screenshot(f"{filename}.png")

        if is_video_enabled():
            save_video_recording(f"{filename}.mp4", scenario_failed())

    # teardown accounts and configs
    teardown_client()

    # search coredumps after every test scenario
    # CI pipeline might fail although all tests are passing
    if coredumps := get_core_dumps():
        try:
            generate_stacktrace(context.title, coredumps)
            test.log("Stacktrace generated!")
        except OSError as err:
            test.log("Exception occured:" + str(err))
    elif scenario_failed():
        test.log("No coredump found!")

    global PREVIOUS_FAIL_RESULT_COUNT, PREVIOUS_ERROR_RESULT_COUNT
    PREVIOUS_FAIL_RESULT_COUNT = test.resultCount("fails")
    PREVIOUS_ERROR_RESULT_COUNT = test.resultCount("errors")


# runs after every scenario
# Order: 2
# server cleanup
@OnScenarioEnd
def hook(context):
    if not is_owncloud_client():
        delete_all_resources()
        permanently_delete_all_resources()
        delete_all_spaces()
        permanently_delete_all_spaces()
    if get_config("ocis"):
        delete_project_spaces()
    else:
        oc.restore_apps_state()
    delete_created_groups()
    delete_created_users()


def teardown_client():
    # Cleanup user accounts from UI for Windows platform
    # It is not needed for Linux so skipping it in order to save CI time
    if is_windows():
        # remove account from UI
        # In Windows, removing only config and sync folders won't help
        # so to work around that, remove the account connection
        close_dialogs()
        close_widgets()
        accounts, _ = Toolbar.get_accounts()
        for account in accounts:
            Toolbar.open_account(account["displayname"])
            AccountSetting.remove_account_connection()
        if accounts:
            squish.waitForObject(AccountConnectionWizard.SERVER_ADDRESS_BOX)

    # Detach (i.e. potentially terminate) all AUTs at the end of a scenario
    for ctx in squish.applicationContextList():
        # get pid before detaching
        pid = ctx.pid
        ctx.detach()
        wait_until_app_killed(pid)

    # clean up config files
    shutil.rmtree(get_config("clientConfigDir"))

    # delete test files/folders
    for entry in os.scandir(get_config("clientRootSyncPath")):
        try:
            if entry.is_file() or entry.is_symlink():
                test.log("Deleting file: " + entry.name)
                os.unlink(prefix_path_namespace(entry.path))
            elif entry.is_dir():
                test.log("Deleting folder: " + entry.name)
                shutil.rmtree(prefix_path_namespace(entry.path))
        except OSError as e:
            test.log(f"Failed to delete '{entry.name}'.\nReason: {e}.")
    # cleanup paths created outside of the temporary directory during the test
    cleanup_created_paths()


def close_dialogs():
    # close the current active dailog if it's not a main client window
    while True:
        active_window = QApplication.activeModalWidget()
        if str(active_window) == "<null>":
            break
        test.log(f"Closing '{active_window.objectName}' window")
        if not active_window.close():
            confirm_dialog = QApplication.activeModalWidget()
            if confirm_dialog.visible:
                squish.clickButton(
                    squish.waitForObject(AccountSetting.CONFIRMATION_YES_BUTTON)
                )


def close_widgets():
    try:
        ch = object.children(squish.waitForObject(AccountSetting.DIALOG_STACK, 500))
        for obj in ch:
            if (
                hasattr(obj, "objectName")
                and obj.objectName
                and obj.objectName != "page"
            ):
                obj.close()
                # if the dialog has a confirmation dialog, confirm it
                confirm_dialog = QApplication.activeModalWidget()
                if str(confirm_dialog) != "<null>" and confirm_dialog.visible:
                    squish.clickButton(
                        squish.waitForObject(AccountSetting.CONFIRMATION_YES_BUTTON)
                    )
    except LookupError:
        # nothing to close if DIALOG_STACK is not found
        # required for client versions <= 5
        pass



from base64 import b64encode
from typing import NamedTuple
import json
import os

from helpers.ConfigHelper import get_config


class User(NamedTuple):
    username: str
    password: str
    displayname: str
    email: str


test_users = {
    "admin": User(
        username="admin",
        password="admin",
        displayname="adminUsername",
        email="admin@example.org",
    ),
    "Alice": User(
        username="Alice",
        password="1234",
        displayname="Alice Hansen",
        email="alice@example.org",
    ),
    "Brian": User(
        username="Brian",
        password="AaBb2Cc3Dd4",
        displayname="Brian Murphy",
        email="brian@example.org",
    ),
    "Carol": User(
        username="Carol",
        password="1234",
        displayname="Carol King",
        email="carol@example.org",
    ),
    "David": User(
        username="David",
        password="1234",
        displayname="David Lopez",
        email="david@example.org",
    ),
}


def get_default_password():
    return "1234"


def basic_auth_header(user=None, password=None):
    if not user and not password:
        user = "admin"
        password = "admin"
    elif not user == "public" and not password:
        password = get_password_for_user(user)

    token = b64encode((f"{user}:{password}").encode()).decode()
    return {"Authorization": "Basic " + token}


def get_user_info(username, attribute):
    if username in test_users:
        return getattr(test_users[username], attribute)
    if attribute == "password":
        return get_default_password()
    raise ValueError(f"Invalid user attribute: {attribute}")


def get_displayname_for_user(username):
    return get_user_info(username, "displayname")


def get_password_for_user(username):
    return get_user_info(username, "password")


def get_username_for_user(username):
    return get_user_info(username, "username")


def init_predefined_users():
    with open(
        os.path.abspath("../" + get_config("predefined_users")), encoding="utf-8"
    ) as f:
        users = json.load(f)
        for key, value in users.items():
            test_users.update(
                {
                    key: User(
                        username=value["username"],
                        password=value["password"],
                        displayname=value["displayname"],
                        email=value["email"],
                    )
                }
            )



import names
import squish
import object  # pylint: disable=redefined-builtin


class SharingDialog:
    SHARE_WITH_COLLABORATOR_INPUT_FIELD = {
        "container": names.qt_tabwidget_stackedwidget_SharingDialogUG_OCC_ShareUserGroupWidget,
        "name": "shareeLineEdit",
        "type": "QLineEdit",
        "visible": 1,
    }
    SUGGESTED_COLLABORATOR = {"type": "QListView", "unnamed": 1, "visible": 1}
    EDIT_PERMISSIONS_CHECKBOX = {
        "container": names.sharingDialogUG_scrollArea_QScrollArea,
        "name": "permissionsEdit",
        "type": "QCheckBox",
        "visible": 1,
    }
    SHARE_PERMISSIONS_CHECKBOX = {
        "container": names.sharingDialogUG_scrollArea_QScrollArea,
        "name": "permissionShare",
        "type": "QCheckBox",
        "visible": 1,
    }
    DELETE_SHARE_BUTTON = {
        "container": names.sharingDialogUG_scrollArea_QScrollArea,
        "name": "deleteShareButton",
        "type": "QToolButton",
        "visible": 1,
    }
    SHARING_DIALOG_CLOSE_BUTTON = {
        "text": "Close",
        "type": "QPushButton",
        "unnamed": 1,
        "visible": 1,
        "window": names.sharingDialog_OCC_ShareDialog,
    }
    SHARING_DIALOG = {
        "type": "QLabel",
        "unnamed": 1,
        "visible": 1,
        "window": names.sharingDialog_OCC_ShareDialog,
    }
    SHARING_DIALOG_ERROR = {"name": "errorLabel", "type": "QLabel", "visible": 1}
    SHARING_DIALOG_CONTRIBUTOR_ROW = {
        "container": names.sharingDialogUG_scrollArea_QScrollArea,
        "name": "sharedWith",
        "type": "QLabel",
        "visible": 1,
    }

    @staticmethod
    def get_available_permission():
        edit_checked = squish.waitForObjectExists(
            SharingDialog.EDIT_PERMISSIONS_CHECKBOX
        ).checked
        share_checked = squish.waitForObjectExists(
            SharingDialog.SHARE_PERMISSIONS_CHECKBOX
        ).checked

        return edit_checked, share_checked

    @staticmethod
    def search_collaborator(collaborator):
        squish.mouseClick(
            squish.waitForObject(SharingDialog.SHARE_WITH_COLLABORATOR_INPUT_FIELD)
        )
        squish.type(
            squish.waitForObject(SharingDialog.SHARE_WITH_COLLABORATOR_INPUT_FIELD),
            collaborator,
        )

    @staticmethod
    def add_collaborator(receiver, permissions, is_group=False, collaborator_count=1):
        SharingDialog.select_collaborator(receiver, is_group)
        permissions_list = permissions.split(",")

        edit_checked, share_checked = SharingDialog.get_available_permission()

        if ("edit" in permissions_list and edit_checked is False) or (
            "edit" not in permissions_list and edit_checked is True
        ):
            squish.clickButton(
                squish.waitForObject(SharingDialog.EDIT_PERMISSIONS_CHECKBOX)
            )
        if ("share" in permissions_list and share_checked is False) or (
            "share" not in permissions_list and share_checked is True
        ):
            squish.clickButton(
                squish.waitForObject(SharingDialog.SHARE_PERMISSIONS_CHECKBOX)
            )

        # wait for share to complete
        # create the copy of the selector to avoid mutating the original
        collaborator_row_selector = SharingDialog.SHARING_DIALOG_CONTRIBUTOR_ROW.copy()
        collaborator_row_selector["occurrence"] = collaborator_count
        squish.waitForObjectExists(collaborator_row_selector)

    @staticmethod
    def get_sharing_dialog_message():
        return str(squish.waitForObjectExists(SharingDialog.SHARING_DIALOG).text)

    @staticmethod
    def get_error_text():
        return str(squish.waitForObjectExists(SharingDialog.SHARING_DIALOG_ERROR).text)

    @staticmethod
    def select_collaborator(receiver, is_group=False):
        suffix = ""
        if is_group:
            suffix = " (group)"

        SharingDialog.search_collaborator(receiver)

        # collaborator name with special characters contains escape characters '\\'
        # in the squish object
        # Example:
        # Actual collaborator name: Speci@l_Name-.+
        # Collaborator name in object: Speci@l\\_Name-\\.+
        escaped_receiver_name = receiver.replace("_", "\\_").replace(".", "\\.")
        squish.mouseClick(
            squish.waitForObjectItem(
                SharingDialog.SUGGESTED_COLLABORATOR,
                escaped_receiver_name + suffix,
            )
        )

    @staticmethod
    def remove_permissions(permissions):
        remove_permissions_list = permissions.split(",")
        (
            is_edit_permission_available,
            is_share_permission_available,
        ) = SharingDialog.get_available_permission()

        if "share" in remove_permissions_list and is_share_permission_available:
            squish.clickButton(
                squish.waitForObject(SharingDialog.SHARE_PERMISSIONS_CHECKBOX)
            )

        if "edit" in remove_permissions_list and is_edit_permission_available:
            squish.clickButton(
                squish.waitForObject(SharingDialog.EDIT_PERMISSIONS_CHECKBOX)
            )

    @staticmethod
    def unshare_with():
        squish.clickButton(squish.waitForObject(SharingDialog.DELETE_SHARE_BUTTON))

    @staticmethod
    def close_sharing_dialog():
        squish.clickButton(
            squish.waitForObject(SharingDialog.SHARING_DIALOG_CLOSE_BUTTON)
        )

    @staticmethod
    def get_collaborators():
        squish.waitForObject(SharingDialog.SHARING_DIALOG_CONTRIBUTOR_ROW)
        return squish.findAllObjects(SharingDialog.SHARING_DIALOG_CONTRIBUTOR_ROW)

    @staticmethod
    def has_edit_permission():
        if not object.exists(SharingDialog.EDIT_PERMISSIONS_CHECKBOX):
            return False
        return squish.waitForObjectExists(
            SharingDialog.EDIT_PERMISSIONS_CHECKBOX
        ).checked

    @staticmethod
    def has_share_permission():
        if not object.exists(SharingDialog.SHARE_PERMISSIONS_CHECKBOX):
            return False
        return squish.waitForObjectExists(
            SharingDialog.SHARE_PERMISSIONS_CHECKBOX
        ).checked

    @staticmethod
    def get_collaborator_name(occurrence=1):
        selector = SharingDialog.SHARING_DIALOG_CONTRIBUTOR_ROW.copy()
        selector["occurrence"] = occurrence
        return str(squish.waitForObjectExists(selector).text)

    @staticmethod
    def is_user_in_suggestion_list(user):
        exists = False
        try:
            squish.waitForObjectItem(SharingDialog.SUGGESTED_COLLABORATOR, user)
            exists = True
        except LookupError as e:
            pass
        return exists



import os
import subprocess
from urllib.parse import urlparse
from os import makedirs
from os.path import exists, join
import test
import psutil
import squish

from helpers.ConfigHelper import get_config, set_config, is_windows, is_owncloud_client
from helpers.ReportHelper import is_video_enabled
from helpers.UserHelper import get_username_for_user
from helpers.api.provisioning import created_users


def substitute_inline_codes(value):
    value = value.replace('%local_server%', get_config('localBackendUrl'))
    value = value.replace('%secure_local_server%', get_config('secureLocalBackendUrl'))
    value = value.replace('%client_root_sync_path%', get_config('clientRootSyncPath'))
    value = value.replace('%current_user_sync_path%', get_config('currentUserSyncPath'))
    value = value.replace(
        '%local_server_hostname%', urlparse(get_config('localBackendUrl')).netloc
    )
    value = value.replace('%home%', get_config('home_dir'))

    return value


def get_client_details(details):
    client_details = {
        'server': '',
        'user': '',
        'password': '',
        'sync_folder': '',
        'oauth': False,
    }
    for row in details:
        row[1] = substitute_inline_codes(row[1])
        if row[0] == 'server':
            client_details.update({'server': row[1]})
        elif row[0] == 'user':
            client_details.update({'user': get_username_for_user(row[1])})
        elif row[0] == 'password':
            if not is_owncloud_client():
                for created_user in created_users.values():
                    if created_user['username'] == client_details['user']:
                        client_details.update({'password': created_user['password']})
                        break
            else:
                client_details.update({'password': row[1]})
        elif row[0] == 'sync_folder':
            client_details.update({'sync_folder': row[1]})
    return client_details


def create_user_sync_path(username):
    # '' at the end adds '/' to the path
    user_sync_path = join(get_config('clientRootSyncPath'), username, '')

    if not exists(user_sync_path):
        makedirs(user_sync_path)

    set_current_user_sync_path(user_sync_path)
    return user_sync_path.replace('\\', '/')


def create_space_path(space='Personal'):
    space_path = join(get_config('currentUserSyncPath'), space, '')
    if not exists(space_path):
        makedirs(space_path)
    return space_path.replace('\\', '/')


def set_current_user_sync_path(sync_path):
    set_config('currentUserSyncPath', sync_path)


def get_resource_path(resource='', user='', space=''):
    sync_path = get_config('currentUserSyncPath')
    if user:
        sync_path = user
    if get_config('ocis'):
        space = space or get_config('syncConnectionName')
        sync_path = join(sync_path, space)
    sync_path = join(get_config('clientRootSyncPath'), sync_path)
    resource = resource.replace(sync_path, '').strip('/').strip('\\')
    if is_windows():
        resource = resource.replace('/', '\\')
    return join(
        sync_path,
        resource,
    )


def get_temp_resource_path(resource_name):
    return join(get_config('tempFolderPath'), resource_name)


def get_current_user_sync_path():
    return get_config('currentUserSyncPath')


def start_client():
    check_keyring()

    squish.startApplication(
        f'{get_config("client_name").lower()} -s'
        + f' --logfile {get_config("clientLogFile")}'
        + ' --logdebug'
        + ' --logflush'
    )
    if is_video_enabled():
        test.startVideoCapture()
    else:
        test.log(
            f'Video recordings reached the maximum limit of {get_config("video_record_limit")}.'
            + 'Skipping video recording...'
        )


def is_app_killed(pid):
    try:
        psutil.Process(pid)
        return False
    except psutil.NoSuchProcess:
        return True


def wait_until_app_killed(pid=0):
    timeout = 5 * 1000
    killed = squish.waitFor(
        lambda: is_app_killed(pid),
        timeout,
    )
    if not killed:
        test.log(f'Application was not terminated within {timeout} milliseconds')


# sometimes the keyring is locked during the test execution, and we need to unlock it
def check_keyring():
    if is_windows():
        return

    if is_keyring_locked():
        test.log('[INFO] Keyring is locked or service is down. Unlocking...')
        wait_until_keyring_unlocked()


def unlock_keyring() -> bool | None:
    if is_windows():
        return None

    password = os.getenv('VNC_PW')
    command = f'echo -n "{password}" | gnome-keyring-daemon -r --unlock'
    stdout, stderr, returncode = run_sys_command(command, True)

    output = ''
    if stdout:
        output = stdout.decode('utf-8')
    if stderr:
        output = stderr.decode('utf-8')
    test.log(f'[INFO] Keyring unlock output: {output}')
    # wait for keyring to unlock
    squish.snooze(1)

    if returncode:
        return False

    return not is_keyring_locked()


def wait_until_keyring_unlocked():
    if is_windows():
        return

    timeout = 10
    unlocked = squish.waitFor(
        lambda: unlock_keyring(),  # pylint: disable=unnecessary-lambda
        timeout * 1000,
    )
    if not unlocked:
        test.fail(f'Timeout. Keyring was not unlocked within {timeout} seconds')


def is_keyring_locked() -> bool | None:
    if is_windows():
        return None

    stdout, stderr, _ = run_sys_command(
        [
            'busctl',
            '--user',
            'get-property',
            'org.freedesktop.secrets',
            '/org/freedesktop/secrets/collection/login',
            'org.freedesktop.Secret.Collection',
            'Locked',
        ]
    )
    output = ''
    if stdout:
        output = stdout.decode('utf-8')
    if stderr:
        output = stderr.decode('utf-8')
    test.log(f'[INFO] Keyring locked status: {output}')
    return not output.strip().endswith('false')


def run_sys_command(command=None, shell=False):
    cmd = subprocess.run(
        command,
        shell=shell,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        check=False,
    )
    return cmd.stdout, cmd.stderr, cmd.returncode
